@@ -32,6 +32,13 @@ struct rsrc_type_info_struct;
 
 typedef struct localctx_struct {
 	int fmt;
+	u8 execomp_mode; // 0 or 1; 0xff=unspecified
+	struct fmtutil_exe_info *ei;
+
+	i64 reloc_tbl_offset;
+	i64 file_hdr_size;
+	i64 num_relocs;
+	i64 end_of_dos_code;
 	i64 ext_header_offset;
 
 	i64 ne_rsrc_tbl_offset;
@@ -499,6 +506,12 @@ static void do_lx_or_le_ext_header(deark *c, lctx *d, i64 pos)
 	de_dbg(c, "data pages offset=%d", (int)d->lx_data_pages_offset);
 }
 
+static void identify_as_dos(deark *c, lctx *d)
+{
+	d->fmt = EXE_FMT_DOS;
+	de_declare_fmt(c, "DOS EXE");
+}
+
 static void do_ext_header(deark *c, lctx *d)
 {
 	u8 buf[4];
@@ -533,26 +546,144 @@ static void do_ext_header(deark *c, lctx *d)
 	}
 
 done:
+	if(d->fmt==0) {
+		if(d->end_of_dos_code == c->infile->len) {
+			identify_as_dos(c, d);
+		}
+	}
+
 	// If we still don't know the format...
-	de_declare_fmt(c, "Unknown EXE format (maybe MS-DOS)");
+	de_declare_fmt(c, "Unknown EXE format (maybe DOS)");
 }
 
-static void do_fileheader(deark *c, lctx *d)
+static void do_reloc_table(deark *c, lctx *d)
 {
-	i64 reloc_tbl_offset;
+	i64 reloc_tbl_endpos;
 
-	reloc_tbl_offset = de_getu16le(24);
-	de_dbg(c, "relocation table offset: %d", (int)reloc_tbl_offset);
+	if(d->num_relocs<1) goto done;
+	if(d->num_relocs > 320*1024) goto done;
+	if(d->reloc_tbl_offset<1) goto done;
+	reloc_tbl_endpos = d->reloc_tbl_offset + d->num_relocs*4;
+	if(reloc_tbl_endpos > d->file_hdr_size) goto done;
+	if(reloc_tbl_endpos > c->infile->len) goto done;
 
-	if(reloc_tbl_offset>=28 && reloc_tbl_offset<64) {
-		de_declare_fmt(c, "MS-DOS EXE");
-		d->fmt = EXE_FMT_DOS;
+	de_dbg(c, "reloc table at %"I64_FMT, d->reloc_tbl_offset);
+	de_dbg_indent(c, 1);
+	de_dbg(c, "%u entries, ends at %"I64_FMT, (UI)d->num_relocs, reloc_tbl_endpos);
+
+	if(c->debug_level>=2) {
+		i64 i;
+		i64 pos = d->reloc_tbl_offset;
+		i64 num_to_dbg = d->num_relocs;
+
+		if(num_to_dbg > 5000) num_to_dbg = 5000;
+
+		for(i=0; i<num_to_dbg; i++) {
+			i64 offs, para;
+
+			offs = de_getu16le_p(&pos);
+			para = de_getu16le_p(&pos);
+			de_dbg(c, "reloc[%u]: %04x:%04x (0x%05x)", (UI)i, (UI)para, (UI)offs,
+				(UI)(para*16+offs));
+		}
 	}
-	else {
-		d->ext_header_offset = de_getu32le(60);
-		de_dbg(c, "extended header offset: %d", (int)d->ext_header_offset);
+
+	de_dbg_indent(c, -1);
+done:
+	;
+}
+
+// Returns 0 only if this is not an EXE file.
+// This function is not expected to extract any files. In execomp mode, it
+// definitely should not extract anything.
+static int do_fileheader(deark *c, lctx *d, i64 pos1)
+{
+	i64 n;
+	i64 lfb, nblocks;
+	i64 pos = pos1;
+	i64 regCS, regIP;
+	int retval = 0;
+	int saved_indent_level;
+	struct de_fourcc sig;
+
+	de_dbg_indent_save(c, &saved_indent_level);
+	de_dbg(c, "DOS file header at %"I64_FMT, pos1);
+	de_dbg_indent(c, 1);
+
+	dbuf_read_fourcc(c->infile, pos, &sig, 2, 0x0);
+	de_dbg(c, "signature: '%s'", sig.id_dbgstr);
+	if(sig.id!=0x4d5aU && sig.id!=0x5a4dU) {
+		de_err(c, "Not an EXE file");
+		goto done;
+	}
+	pos += 2;
+	retval = 1;
+
+	lfb = de_getu16le_p(&pos);
+	de_dbg(c, "length of final block: %u%s", (UI)lfb, ((lfb==0)?" (=512)":""));
+	nblocks = de_getu16le_p(&pos);
+	de_dbg(c, "num blocks: %u", (UI)nblocks);
+
+	d->num_relocs = de_getu16le_p(&pos);
+	de_dbg(c, "num reloc table entries: %u", (UI)d->num_relocs);
+	n = de_getu16le_p(&pos);
+	d->file_hdr_size = n*16;
+	de_dbg(c, "file hdr size: %u ("DE_CHAR_TIMES"16=%"I64_FMT")", (UI)n, d->file_hdr_size);
+
+	n = de_getu16le_p(&pos);
+	de_dbg(c, "min mem: %u ("DE_CHAR_TIMES"16=%"I64_FMT")", (UI)n, (i64)n*16);
+	n = de_getu16le_p(&pos);
+	de_dbg(c, "max mem: %u ("DE_CHAR_TIMES"16=%"I64_FMT")", (UI)n, (i64)n*16);
+
+	n = de_geti16le_p(&pos);
+	de_dbg(c, "regSS: %d", (int)n);
+	n = de_getu16le_p(&pos);
+	de_dbg(c, "regSP: %u", (UI)n);
+
+	n = de_getu16le_p(&pos);
+	de_dbg(c, "checksum: 0x%04x", (UI)n);
+
+	regIP = de_getu16le_p(&pos);
+	de_dbg(c, "regIP: %u", (UI)regIP);
+	regCS = de_geti16le_p(&pos);
+	de_dbg(c, "regCS: %d ("DE_CHAR_TIMES"16=%d)", (int)regCS, (int)(16*regCS));
+
+	d->reloc_tbl_offset = de_getu16le_p(&pos);
+	de_dbg(c, "reloc table offset: %d%s", (int)d->reloc_tbl_offset,
+		(d->num_relocs==0)?" (no entries)":"");
+
+	n = de_getu16le_p(&pos);
+	de_dbg(c, "overlay indicator: %u", (UI)n);
+
+	d->end_of_dos_code = nblocks*512;
+	if(lfb>=1 && lfb<=511) {
+		d->end_of_dos_code = d->end_of_dos_code - 512 + lfb;
+	}
+
+	de_dbg(c, "DOS executable code: start=%"I64_FMT", len=%"I64_FMT", end=%"I64_FMT"",
+		d->file_hdr_size, d->end_of_dos_code-d->file_hdr_size, d->end_of_dos_code);
+	de_dbg(c, "DOS entry point: %"I64_FMT, d->file_hdr_size + 16*regCS + regIP);
+	if(d->end_of_dos_code < c->infile->len) {
+		de_dbg(c, "bytes after DOS executable code: %"I64_FMT, c->infile->len - d->end_of_dos_code);
+	}
+
+	de_dbg_indent(c, -1);
+
+	do_reloc_table(c, d);
+
+	if(d->reloc_tbl_offset>=28 && d->reloc_tbl_offset<64) {
+		identify_as_dos(c, d);
+	}
+
+	if(d->fmt!=EXE_FMT_DOS) {
+		d->ext_header_offset = de_getu32le(pos1+60);
+		de_dbg(c, "extended header offset: %"I64_FMT, d->ext_header_offset);
 		do_ext_header(c, d);
 	}
+
+done:
+	de_dbg_indent_restore(c, saved_indent_level);
+	return retval;
 }
 
 static void do_decode_ddb(deark *c, lctx *d, i64 pos1, i64 len, de_finfo *fi)
@@ -1307,14 +1438,121 @@ static void do_lx_or_le_rsrc_tbl(deark *c, lctx *d)
 	}
 }
 
+static void extract_zip_from_sfx(deark *c, lctx *d,
+	struct fmtutil_specialexe_detection_data *edd)
+{
+	de_module_params *mparams = NULL;
+
+	if(!d->ei) return;
+	de_dbg(c, "attempting zip extraction");
+	de_dbg_indent(c, 1);
+
+	mparams = de_malloc(c, sizeof(de_module_params));
+	mparams->in_params.codes = "R";
+	mparams->in_params.obj1 = (void*)edd;
+
+	de_run_module_by_id_on_slice(c, "zip", mparams, d->ei->f, 0, d->ei->f->len);
+
+	if(!(mparams->out_params.flags & 0x1)) {
+		de_warn(c, "This look like a self-extracting ZIP file, but an attempt to "
+			"extract the plain ZIP file failed.");
+	}
+	de_free(c, mparams);
+	de_dbg_indent(c, -1);
+}
+
+static void do_exesfx(deark *c, lctx *d)
+{
+	struct fmtutil_specialexe_detection_data edd;
+
+	if(!d->ei) return;
+	de_zeromem(&edd, sizeof(struct fmtutil_specialexe_detection_data));
+
+	fmtutil_detect_exesfx(c, d->ei, &edd);
+
+	if(edd.zip_eocd_found && !edd.detected_fmt) {
+		// Possibly a ZIP SFX that we don't know how to handle
+		de_info(c, "Note: This might be a self-extracting ZIP file (try \"-m zip\").");
+		goto done;
+	}
+
+	if(!edd.detected_fmt) goto done;
+
+	de_dbg(c, "detected self-extracting exe: %s", edd.detected_fmt_name);
+	if(edd.payload_valid) {
+		dbuf_create_file_from_slice(d->ei->f, edd.payload_pos, edd.payload_len, edd.payload_file_ext, NULL, 0);
+	}
+	else if(edd.detected_fmt==DE_SPECIALEXEFMT_ZIPSFX) {
+		// ZIP SFX generally needs special processing
+		extract_zip_from_sfx(c, d, &edd);
+	}
+
+done:
+	;
+}
+
+// Used in execomp mode.
+static void do_execomp_decompress(deark *c, lctx *d, struct fmtutil_specialexe_detection_data *edd)
+{
+	if(!edd->detected_fmt_name) return;
+	if(!edd->modname) {
+		de_info(c, "Note: File seems to be compressed with %s, but that's "
+			"not a supported format.", edd->detected_fmt_name);
+		return;
+	}
+
+	de_dbg(c, "attempting %s decompression", edd->modname);
+	de_dbg_indent(c, 1);
+	de_run_module_by_id_on_slice2(c, edd->modname, NULL, c->infile, 0, c->infile->len);
+	de_dbg_indent(c, -1);
+}
+
+// Based on d->execomp_mode, either just check for executable compression,
+// or try to decompress.
+static void do_execomp(deark *c, lctx *d)
+{
+	struct fmtutil_specialexe_detection_data edd;
+
+	if(d->execomp_mode!=1) {
+		if((!c->show_infomessages) && (c->debug_level<1)) return;
+	}
+
+	if(!d->ei) {
+		d->ei = de_malloc(c, sizeof(struct fmtutil_exe_info));
+		fmtutil_collect_exe_info(c, c->infile, d->ei);
+	}
+
+	de_zeromem(&edd, sizeof(struct fmtutil_specialexe_detection_data));
+	fmtutil_detect_execomp(c, d->ei, &edd);
+	if(!edd.detected_fmt) return;
+	de_dbg(c, "detected executable compression: %s", edd.detected_fmt_name);
+
+	if(d->execomp_mode==1) {
+		do_execomp_decompress(c, d, &edd);
+	}
+	else if(d->execomp_mode!=0) {
+		if(edd.modname) {
+			de_info(c, "Note: File seems to be compressed with %s. Use \"-m %s\" "
+				"or \"-opt execomp\" to attempt decompression.",
+				edd.detected_fmt_name, edd.modname);
+		}
+	}
+}
+
 static void de_run_exe(deark *c, de_module_params *mparams)
 {
 	lctx *d = NULL;
-	int zip_eocd_found;
 
 	d = de_malloc(c, sizeof(lctx));
 
-	do_fileheader(c, d);
+	d->execomp_mode = (u8)de_get_ext_option_bool(c, "execomp", 0xff);
+
+	if(!do_fileheader(c, d, 0)) goto done;
+
+	if(d->execomp_mode==1) {
+		do_execomp(c, d);
+		goto done;
+	}
 
 	if((d->fmt==EXE_FMT_PE32 || d->fmt==EXE_FMT_PE32PLUS) && d->pe_sections_offset>0) {
 		do_pe_section_table(c, d);
@@ -1326,29 +1564,39 @@ static void de_run_exe(deark *c, de_module_params *mparams)
 		do_lx_or_le_rsrc_tbl(c, d);
 	}
 
-	if(c->detection_data && c->detection_data->zip_eocd_looked_for) {
-		// Note: It isn't necessarily possible to get here - It depends on the details
-		// of how other modules' identify() functions work.
-		zip_eocd_found = (int)c->detection_data->zip_eocd_found;
-	}
-	else {
-		i64 zip_eocd_pos = 0;
-		zip_eocd_found = fmtutil_find_zip_eocd(c, c->infile, &zip_eocd_pos);
-	}
-	if(zip_eocd_found) {
-		de_info(c, "Note: This might be a self-extracting ZIP file (try \"-m zip\").");
+	if(!d->ei) {
+		d->ei = de_malloc(c, sizeof(struct fmtutil_exe_info));
+		fmtutil_collect_exe_info(c, c->infile, d->ei);
 	}
+	do_exesfx(c, d);
+	do_execomp(c, d);
 
-	de_free(c, d);
+done:
+	if(d) {
+		de_free(c, d->ei);
+		de_free(c, d);
+	}
 }
 
+// Note - This needs to be coordinated with de_identify_zip, so that self-extracting
+// ZIP files are identified as exe and not zip (assuming all modules are enabled).
 static int de_identify_exe(deark *c)
 {
-	u8 buf[2];
-	de_read(buf, 0, 2);
+	UI x;
+	int sigMZ;
 
-	if(buf[0]=='M' && buf[1]=='Z') return 80;
-	return 0;
+	x = (UI)de_getu16be(0);
+	if(x==0x4d5aU) { sigMZ = 1; }
+	else if(x==0x5a4d) { sigMZ = 0; }
+	else { return 0; }
+	x = (UI)de_getu16le(2); // lenFinal
+	if(x>511) return 0;
+	return sigMZ ? 80 : 50;
+}
+
+static void de_help_exe(deark *c)
+{
+	de_msg(c, "-opt execomp : Try to decompress compressed files");
 }
 
 void de_module_exe(deark *c, struct deark_module_info *mi)
@@ -1357,4 +1605,5 @@ void de_module_exe(deark *c, struct deark_module_info *mi)
 	mi->desc = "EXE executable (PE, NE, etc.)";
 	mi->run_fn = de_run_exe;
 	mi->identify_fn = de_identify_exe;
+	mi->help_fn = de_help_exe;
 }
