@@ -467,7 +467,7 @@ static void render_plaintext_char(deark *c, lctx *d, u8 ch,
 
 struct plaintext_ext_ctx {
 	int header_ok;
-	int ok_to_render; // If 0, something's wrong, and we should't draw the pixels
+	int ok_to_render; // If 0, something's wrong, and we shouldn't draw the pixels
 	i64 textarea_xpos_in_pixels, textarea_ypos_in_pixels;
 	i64 textarea_xsize_in_pixels, textarea_ysize_in_pixels;
 	i64 text_width_in_chars;
@@ -725,6 +725,23 @@ static void do_mgkiptc_extension(deark *c, lctx *d, i64 pos)
 	dbuf_close(tmpf);
 }
 
+static void do_giflite_extension(deark *c, lctx *d, i64 pos1)
+{
+	i64 sub_block_len;
+	i64 pos = pos1;
+	UI vmajor, vminor;
+	i64 olen;
+
+	sub_block_len = (i64)de_getbyte_p(&pos);
+	if(sub_block_len<2) return;
+	vmajor = (UI)de_getbyte_p(&pos);
+	vminor = (UI)de_getbyte_p(&pos);
+	de_dbg(c, "GIFLITE ver: %u.%02u", vmajor, vminor);
+	if(sub_block_len<6) return;
+	olen = de_getu32le_p(&pos);
+	de_dbg(c, "orig file size: %"I64_FMT, olen);
+}
+
 static void do_unknown_extension(deark *c, lctx *d, i64 pos)
 {
 	dbuf *tmpf = NULL;
@@ -771,6 +788,9 @@ static void do_application_extension(deark *c, lctx *d, i64 pos)
 	else if(!de_memcmp(app_id, "MGKIPTC0000", 11)) {
 		do_mgkiptc_extension(c, d, pos);
 	}
+	else if(!de_memcmp(app_id, "GIFLITE    ", 11)) {
+		do_giflite_extension(c, d, pos);
+	}
 	else {
 		do_unknown_extension(c, d, pos);
 	}
@@ -1261,7 +1281,7 @@ static void de_help_gif(deark *c)
 {
 	de_msg(c, "-opt gif:raw : Extract individual component images");
 	de_msg(c, "-opt gif:dumpplaintext : Also extract plain text extensions to text files");
-	de_msg(c, "-opt gif:dumpscreen : Also extact the final \"screen\" contents");
+	de_msg(c, "-opt gif:dumpscreen : Also extract the final \"screen\" contents");
 }
 
 void de_module_gif(deark *c, struct deark_module_info *mi)
