@@ -43,12 +43,6 @@ typedef struct localctx_struct {
 	struct de_crcobj *crco;
 } lctx;
 
-static void our_writelistener_cb(dbuf *f, void *userdata, const u8 *buf, i64 buf_len)
-{
-	struct de_crcobj *crco = (struct de_crcobj*)userdata;
-	de_crcobj_addbuf(crco, buf, buf_len);
-}
-
 static void do_extract_member(deark *c, lctx *d, struct member_data *md)
 {
 	de_finfo *fi = NULL;
@@ -80,7 +74,7 @@ static void do_extract_member(deark *c, lctx *d, struct member_data *md)
 		de_crcobj_addslice(d->crco, c->infile, md->pos_in_bytes+18, md->len_in_bytes_withpadding-18);
 	}
 	else {
-		dbuf_set_writelistener(outf, our_writelistener_cb, (void*)d->crco);
+		dbuf_set_writelistener(outf, de_writelistener_for_crc, (void*)d->crco);
 		dbuf_copy(c->infile, md->pos_in_bytes, md->len_in_bytes_nopadding, outf);
 		// CRC calculation includes padding bytes:
 		de_crcobj_addslice(d->crco, c->infile,
@@ -248,7 +242,7 @@ static void de_run_lbr(deark *c, de_module_params *mparams)
 	d = de_malloc(c, sizeof(lctx));
 	d->input_encoding = de_get_input_encoding(c, NULL, DE_ENCODING_ASCII);
 
-	d->crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_CCITT);
+	d->crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_XMODEM);
 
 	// Read directory
 	if(!do_entry(c, d, pos, 1)) goto done;
@@ -564,6 +558,7 @@ static void de_run_squeeze(deark *c, de_module_params *mparams)
 	// We have to decompress the file before we can find the timestamp. That's
 	// why we decompress to a membuf.
 	outf_tmp = dbuf_create_membuf(c, 0, 0);
+	dbuf_enable_wbuffer(outf_tmp);
 
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
 	dcmpri.f = c->infile;
@@ -580,6 +575,7 @@ static void de_run_squeeze(deark *c, de_module_params *mparams)
 	tlp.dcmpro = &dcmpro;
 	tlp.dres = &dres;
 	de_dfilter_decompress_two_layer(c, &tlp);
+	dbuf_flush(dcmpro.f);
 
 	if(dres.bytes_consumed_valid) {
 		de_dbg(c, "compressed data size: %"I64_FMT", ends at %"I64_FMT, dres.bytes_consumed,
@@ -675,6 +671,7 @@ static void decompress_crunch_v1(deark *c, struct crunch_ctx *crunchctx, i64 pos
 	fi->original_filename_flag = 1;
 
 	outf = dbuf_create_output_file(c, NULL, fi, 0x0);
+	dbuf_enable_wbuffer(outf);
 	dbuf_set_writelistener(outf, crunch_writelistener_cb, (void*)crunchctx);
 
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
@@ -695,6 +692,7 @@ static void decompress_crunch_v1(deark *c, struct crunch_ctx *crunchctx, i64 pos
 	tlp.dcmpro = &dcmpro;
 	tlp.dres = &dres;
 	de_dfilter_decompress_two_layer(c, &tlp);
+	dbuf_flush(dcmpro.f);
 
 	if(dres.errcode) {
 		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
@@ -836,6 +834,7 @@ static void decompress_crlzh(deark *c, struct crlzh_ctx *crlzhctx, i64 pos1)
 	fi->original_filename_flag = 1;
 
 	outf = dbuf_create_output_file(c, NULL, fi, 0x0);
+	dbuf_enable_wbuffer(outf);
 	dbuf_set_writelistener(outf, crlzh_writelistener_cb, (void*)crlzhctx);
 
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
@@ -854,6 +853,7 @@ static void decompress_crlzh(deark *c, struct crlzh_ctx *crlzhctx, i64 pos1)
 	lh1p.history_fill_val = 0x20;
 
 	fmtutil_lh1_codectype1(c, &dcmpri, &dcmpro, &dres, (void*)&lh1p);
+	dbuf_flush(dcmpro.f);
 
 	if(dres.errcode) {
 		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
