@@ -11,6 +11,8 @@ DE_DECLARE_MODULE(de_module_lha);
 DE_DECLARE_MODULE(de_module_swg);
 DE_DECLARE_MODULE(de_module_car_lha);
 DE_DECLARE_MODULE(de_module_arx);
+DE_DECLARE_MODULE(de_module_ar001);
+DE_DECLARE_MODULE(de_module_lharc_sfx_com);
 
 #define MAX_SUBDIR_LEVEL 32
 
@@ -91,11 +93,12 @@ struct member_data {
 
 typedef struct localctx_struct {
 	de_encoding input_encoding;
+	int lhark_policy; // -1=detect, 0=no, 1=yes
+	int lhark_req;
 	u8 hlev_of_first_member;
 	u8 swg_fmt;
-	u8 lhark_fmt;
-	u8 lh7_success_flag;
-	u8 lh7_failed_flag;
+	u8 lh7_success_flag; // currently unused
+	u8 lh7_failed_flag; // currently unused
 	u8 trailer_found;
 	int member_count;
 	i64 trailer_pos;
@@ -733,6 +736,104 @@ static void decompress_lh5x_internal(deark *c, lctx *d,
 	fmtutil_decompress_lh5x(c, dcmpri, dcmpro, dres, &lzhparams);
 }
 
+static int decompress_lh5x_dry_run(deark *c, lctx *d, struct member_data *md,
+	struct de_dfilter_in_params *dcmpri, int fmt)
+{
+	int retval = 0;
+	dbuf *outf = NULL;
+	struct de_crcobj *crco = NULL;
+	u32 crc_calc;
+	int old_debug_level;
+	struct de_dfilter_out_params dcmpro;
+	struct de_dfilter_results dres;
+
+	// Make a "dummy" dbuf to write to, which doesn't store the data, but
+	// tracks the size and CRC.
+	outf = dbuf_create_custom_dbuf(c, 0, 0);
+	dbuf_enable_wbuffer(outf);
+	crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_ARC);
+	dbuf_set_writelistener(outf, de_writelistener_for_crc, crco);
+
+	de_dfilter_init_objects(c, NULL, &dcmpro, &dres);
+	dcmpro.f = outf;
+	dcmpro.len_known = 1;
+	dcmpro.expected_len = md->orig_size;
+
+	old_debug_level = c->debug_level;
+	c->debug_level = 0; // hack
+	decompress_lh5x_internal(c, d, dcmpri, &dcmpro, &dres, fmt);
+	c->debug_level = old_debug_level;
+	dbuf_flush(outf);
+
+	if(dres.errcode) goto done;
+	if(outf->len != md->orig_size) goto done;
+	// Note: Another possible test would be if
+	//  (dres.bytes_consumed == md->compressed_data_len).
+	crc_calc = de_crcobj_getval(crco);
+	if(crc_calc != md->crc16) goto done;
+	retval = 1;
+
+done:
+	dbuf_close(outf);
+	de_crcobj_destroy(crco);
+	return retval;
+}
+
+// Sets d->lhark_policy.
+// This detection is slow, so we only do it for the first lh7 member in a file,
+// and assume all other lh7 members use the same format.
+static void detect_lhark(deark *c, lctx *d, struct member_data *md,
+	struct de_dfilter_in_params *dcmpri)
+{
+	int ret;
+	int ok = 0;
+	const char *fmt_name;
+
+	if(d->lhark_policy>=0) goto done; // shouldn't get here
+	if(d->lhark_req>=0) {
+		d->lhark_policy = d->lhark_req; // shouldn't get here
+		goto done;
+	}
+
+	de_dbg(c, "[detecting lh7 format]");
+	de_dbg_indent(c, 1);
+
+	if(md->hlev != 1) {
+		d->lhark_policy = 0;
+		ok = 1;
+		goto done;
+	}
+
+	ret = decompress_lh5x_dry_run(c, d, md, dcmpri, DE_LH5X_FMT_LH7);
+	if(ret) {
+		d->lhark_policy = 0;
+		ok = 1;
+		goto done;
+	}
+
+	ret = decompress_lh5x_dry_run(c, d, md, dcmpri, DE_LH5X_FMT_LHARK);
+	if(ret) {
+		d->lhark_policy = 1;
+		ok = 1;
+		goto done;
+	}
+
+	d->lhark_policy = 0;
+
+done:
+	if(ok) {
+		if(d->lhark_policy>0)
+			fmt_name = "LHARK";
+		else
+			fmt_name = "standard lh7";
+	}
+	else {
+		fmt_name = "unknown, assuming standard lh7";
+	}
+	de_dbg(c, "detected lh7 format: %s", fmt_name);
+	de_dbg_indent(c, -1);
+}
+
 // Compression method will be selected based on id_raw[3] (which
 // should be '4'...'8'), etc.
 static void decompress_lh5x_auto(deark *c, lctx *d, struct member_data *md,
@@ -749,7 +850,10 @@ static void decompress_lh5x_auto(deark *c, lctx *d, struct member_data *md,
 		fmt = DE_LH5X_FMT_LH6;
 		break;
 	case '7':
-		if(d->lhark_fmt) {
+		if(d->lhark_policy<0) {
+			detect_lhark(c, d, md, dcmpri);
+		}
+		if(d->lhark_policy>0) {
 			fmt = DE_LH5X_FMT_LHARK;
 		}
 		else {
@@ -777,7 +881,7 @@ static void decompress_lz5(deark *c, lctx *d, struct member_data *md,
 	struct de_dfilter_in_params *dcmpri, struct de_dfilter_out_params *dcmpro,
 	struct de_dfilter_results *dres)
 {
-	fmtutil_decompress_szdd(c, dcmpri, dcmpro, dres, 0x1);
+	fmtutil_decompress_lzss1(c, dcmpri, dcmpro, dres, 0x2);
 }
 
 struct cmpr_meth_array_item {
@@ -794,7 +898,7 @@ static const struct cmpr_meth_array_item cmpr_meth_arr[] = {
 	{ 0x00, CODE_lhd, "directory", NULL },
 	{ 0x00, CODE_lh0, "uncompressed", decompress_uncompressed },
 	{ 0x00, CODE_lh1, "LZ77-4K, adaptive Huffman", decompress_lh1 },
-	{ 0x00, CODE_lh4, NULL, decompress_lh5x_auto },
+	{ 0x00, CODE_lh4, "LZ77-4K, static Huffman", decompress_lh5x_auto },
 	{ 0x00, CODE_lh5, "LZ77-8K, static Huffman", decompress_lh5 },
 	{ 0x00, CODE_lh6, "LZ77-32K, static Huffman", decompress_lh5x_auto },
 	{ 0x00, CODE_lh7, NULL, decompress_lh5x_auto },
@@ -865,12 +969,6 @@ static void get_cmpr_meth_info(const u8 idbuf[5], struct cmpr_meth_info *cmi)
 	}
 }
 
-static void our_writelistener_cb(dbuf *f, void *userdata, const u8 *buf, i64 buf_len)
-{
-	struct de_crcobj *crco = (struct de_crcobj*)userdata;
-	de_crcobj_addbuf(crco, buf, buf_len);
-}
-
 static void do_extract_file(deark *c, lctx *d, struct member_data *md)
 {
 	de_finfo *fi = NULL;
@@ -926,8 +1024,9 @@ static void do_extract_file(deark *c, lctx *d, struct member_data *md)
 	fi->original_filename_flag = 1;
 
 	outf = dbuf_create_output_file(c, NULL, fi, 0x0);
+	dbuf_enable_wbuffer(outf);
 	de_crcobj_reset(d->crco);
-	dbuf_set_writelistener(outf, our_writelistener_cb, (void*)d->crco);
+	dbuf_set_writelistener(outf, de_writelistener_for_crc, (void*)d->crco);
 
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
 	dcmpri.f = c->infile;
@@ -943,6 +1042,7 @@ static void do_extract_file(deark *c, lctx *d, struct member_data *md)
 	if(md->cmi->decompressor) {
 		md->cmi->decompressor(c, d, md, &dcmpri, &dcmpro, &dres);
 	}
+	dbuf_flush(dcmpro.f);
 
 	if(dres.errcode) {
 		de_err(c, "%s: Decompression failed: %s", ucstring_getpsz_d(md->fullfilename),
@@ -1406,7 +1506,8 @@ static void do_run_lha_internal(deark *c, de_module_params *mparams, int is_swg)
 	d = de_malloc(c, sizeof(lctx));
 	if(is_swg) d->swg_fmt = 1;
 
-	d->lhark_fmt = (u8)de_get_ext_option_bool(c, "lha:lhark", 0);
+	d->lhark_req = de_get_ext_option_bool(c, "lha:lhark", -1);
+	d->lhark_policy = d->lhark_req;
 
 	// It's not really safe to guess CP437, because Japanese-encoded (CP932?)
 	// filenames are common.
@@ -1437,13 +1538,6 @@ done:
 
 	destroy_member_data(c, md);
 	if(d) {
-		if(!d->lhark_fmt && d->hlev_of_first_member==1 && d->lh7_failed_flag &&
-			!d->lh7_success_flag)
-		{
-			de_info(c, "Note: 'lh7' decompression failed. Maybe this file uses "
-				"LHARK compression. Try \"-opt lha:lhark\".");
-		}
-
 		de_crcobj_destroy(d->crco);
 		de_free(c, d);
 	}
@@ -1495,7 +1589,8 @@ static int de_identify_lha(deark *c)
 	}
 
 	if(de_input_file_has_ext(c, "lzh") ||
-		de_input_file_has_ext(c, "lha"))
+		de_input_file_has_ext(c, "lha") ||
+		((b[4]=='z') && de_input_file_has_ext(c, "lzs")))
 	{
 		has_ext = 1;
 	}
@@ -1506,7 +1601,7 @@ static int de_identify_lha(deark *c)
 
 static void de_help_lha(deark *c)
 {
-	de_msg(c, "-opt lha:lhark : Enable LHARK mode (for 'lh7' compression)");
+	de_msg(c, "-opt lha:lhark=<0|1> : LHARK mode (for 'lh7' compression)");
 }
 
 void de_module_lha(deark *c, struct deark_module_info *mi)
@@ -1741,7 +1836,7 @@ static void arx_recalc_lh1(deark *c, struct arx_ctx *d, struct arx_member_data *
 	struct de_dfilter_results dres;
 
 	outf = dbuf_create_custom_dbuf(c, md->unc_data_len, 0);
-	dbuf_set_writelistener(outf, our_writelistener_cb, (void*)d->crco);
+	dbuf_set_writelistener(outf, de_writelistener_for_crc, (void*)d->crco);
 
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
 	dcmpri.f = c->infile;
@@ -1918,3 +2013,335 @@ void de_module_arx(deark *c, struct deark_module_info *mi)
 	mi->run_fn = de_run_arx;
 	mi->identify_fn = de_identify_arx;
 }
+
+
+/////////////////////// ar (Haruhiko Okumura) version "ar001"
+
+static void do_check_ar001_header_crc(deark *c, lctx *d, struct member_data *md, i64 basic_hdr_size)
+{
+	//if(!md->have_hdr_crc_reported) return;
+	de_crcobj_reset(d->crco);
+
+	// Everything before the CRC field:
+	de_crcobj_addslice(d->crco, c->infile, md->member_pos+2, basic_hdr_size);
+
+	md->hdr_crc_calc = de_crcobj_getval(d->crco);
+	de_dbg(c, "header crc (calculated): 0x%04x", (UI)md->hdr_crc_calc);
+	if(md->hdr_crc_calc != md->hdr_crc_reported) {
+		de_err(c, "Wrong header CRC: reported=0x%04x, calculated=0x%04x",
+				(UI)md->hdr_crc_reported, (UI)md->hdr_crc_calc);
+	}
+}
+
+// Caller allocates and initializes md.
+// If the member was successfully parsed, sets md->total_size and returns nonzero.
+static int do_read_ar001_member(deark *c, lctx *d, struct member_data *md)
+{
+	int saved_indent_level;
+	int retval = 0;
+	i64 pos1 = md->member_pos;
+	i64 pos = pos1;
+	i64 basic_hdr_size;
+	i64 fnlen;
+	i64 first_ext_hdr_size;
+	UI cmpr_method;
+	u8 cmpr_meth_raw[5];
+
+	de_dbg_indent_save(c, &saved_indent_level);
+	de_dbg(c, "member at %"I64_FMT, pos1);
+	de_dbg_indent(c, 1);
+
+	md->hlev = 0; // (hack)
+	basic_hdr_size = de_getu16le_p(&pos);
+	if(basic_hdr_size==0) {
+		de_dbg(c, "end of archive");
+		goto done;
+	}
+	de_dbg(c, "basic header size: %u", (UI)basic_hdr_size);
+	if(basic_hdr_size<18) goto done;
+	fnlen = basic_hdr_size-18;
+
+	cmpr_method = (UI)de_getu16le_p(&pos);
+
+	de_zeromem(cmpr_meth_raw, 5);
+	switch(cmpr_method) {
+	case 0: de_memcpy(cmpr_meth_raw, (const void*)"-lh0-", 5); break; // (hack)
+	case 1: de_memcpy(cmpr_meth_raw, (const void*)"-lh4-", 5); break; // ...
+	}
+	md->cmi = de_malloc(c, sizeof(struct cmpr_meth_info));
+	get_cmpr_meth_info(cmpr_meth_raw, md->cmi);
+	de_dbg(c, "cmpr method: %u (%s)", cmpr_method, md->cmi->descr);
+
+	pos += 1; // file type
+
+	// timestamp: This is the time the file was added to the archive, not its
+	// last-modified timestamp, so it's not very useful.
+	pos += 5;
+
+	md->compressed_data_len = de_getu32le_p(&pos);
+	de_dbg(c, "compressed size: %"I64_FMT, md->compressed_data_len);
+
+	md->orig_size = de_getu32le_p(&pos);
+	de_dbg(c, "original size: %"I64_FMT, md->orig_size);
+
+	md->crc16 = (u32)de_getu16le_p(&pos);
+	de_dbg(c, "crc16 (reported): 0x%04x", (UI)md->crc16);
+
+	read_filename_hlev0(c, d, md, pos, fnlen);
+	pos += fnlen;
+	make_fullfilename(c, d, md);
+
+	md->hdr_crc_reported = (UI)de_getu16le_p(&pos);
+	de_dbg(c, "basic header crc (reported): 0x%04x", (UI)md->hdr_crc_reported);
+
+	do_check_ar001_header_crc(c, d, md, basic_hdr_size);
+
+	first_ext_hdr_size = de_getu16le_p(&pos);
+	de_dbg(c, "first ext header size: %"I64_FMT, first_ext_hdr_size);
+	if(first_ext_hdr_size) {
+		// The ar001 software never uses this feature, so I'm not going to try to
+		// support it.
+		de_err(c, "Files with extended headers aren't supported");
+		goto done;
+	}
+
+	md->total_size = pos + md->compressed_data_len - pos1;
+	retval = 1;
+
+	md->compressed_data_pos = pos;
+
+	de_dbg(c, "member data at %"I64_FMT", len=%"I64_FMT,
+		md->compressed_data_pos, md->compressed_data_len);
+
+	if(!md->cmi->decompressor) {
+		de_err(c, "%s: Unsupported compression method: %u",
+			ucstring_getpsz_d(md->fullfilename), cmpr_method);
+		goto done;
+	}
+
+	de_dbg_indent(c, 1);
+	do_extract_file(c, d, md);
+
+done:
+	de_dbg_indent_restore(c, saved_indent_level);
+	return retval;
+}
+
+static void de_run_ar001(deark *c, de_module_params *mparams)
+{
+	lctx *d = NULL;
+	i64 pos;
+	struct member_data *md = NULL;
+
+	d = de_malloc(c, sizeof(lctx));
+	d->input_encoding = de_get_input_encoding(c, NULL, DE_ENCODING_ASCII);
+
+	d->crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_IBMSDLC);
+
+	pos = 0;
+	while(1) {
+		if(pos >= c->infile->len) break;
+
+		md = de_malloc(c, sizeof(struct member_data));
+		md->encoding = d->input_encoding;
+		md->member_pos = pos;
+		if(!do_read_ar001_member(c, d, md)) goto done;
+		if(md->total_size<1) goto done;
+
+		d->member_count++;
+		pos += md->total_size;
+
+		destroy_member_data(c, md);
+		md = NULL;
+	}
+
+done:
+	destroy_member_data(c, md);
+	if(d) {
+		de_crcobj_destroy(d->crco);
+		de_free(c, d);
+	}
+}
+
+static int slice_is_printable_ascii(dbuf *f, i64 pos, i64 len)
+{
+	i64 i;
+
+	for(i=0; i<len; i++) {
+		u8 b;
+
+		b = dbuf_getbyte(f, pos+i);
+		if(b<32 || b>126) return 0;
+	}
+	return 1;
+}
+
+static int de_identify_ar001(deark *c)
+{
+	i64 n;
+	i64 bhlen;
+	i64 nlen;
+	u32 bhcrc_r, bhcrc_c;
+	u8 b;
+	int conf = 0;
+	struct de_crcobj *crco = NULL;
+
+	bhlen = de_getu16le(0); // basic header size
+	if(bhlen<(18+1) || bhlen>(18+1024)) goto done;
+	if(c->infile->len < bhlen+8) goto done;
+	nlen = bhlen-18;
+	n = de_getu16le(2); // cmpr method
+	if(n>1) goto done;
+	b = de_getbyte(4); // file type
+	if(b!=0) goto done; // 1 (text) is also defined, but not used by ar001
+	n = de_getu16le(2+bhlen+2); // first ext hdr len
+	if(n!=0) goto done;
+	if(!slice_is_printable_ascii(c->infile, 20, nlen)) goto done;
+	bhcrc_r = (u32)de_getu16le(2+bhlen);
+	crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_IBMSDLC);
+	de_crcobj_addslice(crco, c->infile, 2, bhlen);
+	bhcrc_c = de_crcobj_getval(crco);
+	if(bhcrc_c == bhcrc_r) conf = 91;
+
+done:
+	if(crco) de_crcobj_destroy(crco);
+	return conf;
+}
+
+void de_module_ar001(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "ar001";
+	mi->desc = "ar001 archive (Okumura)";
+	mi->run_fn = de_run_ar001;
+	mi->identify_fn = de_identify_ar001;
+}
+
+// **************************************************************************
+// LHarc & LArc SFX - COM format
+// **************************************************************************
+// Note that EXE SFX format is handled by the exe module.
+
+struct lhasfx_context {
+	u8 errflag;
+	u8 need_errmsg;
+	int sfx_container_is_larc;
+	i64 payload_offs;
+};
+
+static int looks_like_lharc_sfx_com(deark *c, int *is_larc)
+{
+	u64 v;
+	i64 pos = 0;
+	u8 b;
+
+	b = de_getbyte_p(&pos);
+	if(b!=0xeb) return 0;
+	b = de_getbyte_p(&pos);
+	pos += (i64)b;
+	// I don't know how good this test is, but I don't trust the text signature.
+	// It's not formatted consistently. And because the source code was released,
+	// who knows what weirdness is out there?
+	if(b==0x60 || b==0x6c) {
+		v = dbuf_getu64be(c->infile, pos);
+		if(v==0xfcbc0001bb0601e8ULL) {
+			return 1;
+		}
+	}
+	else if(b==0x1c) {
+		v = dbuf_getu64be(c->infile, pos);
+		if(v==0xfc8cc8030602018eULL) {
+			*is_larc = 1;
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+// Probe for LHarc (v1.x) or LArc data
+static int is_lharc_data_at(deark *c, i64 pos, i64 *pfoundpos)
+{
+	u8 b[5];
+
+	if(pos+21 > c->infile->len) return 0;
+	de_read(b, pos+2, sizeof(b));
+	if(b[0]!='-' || b[1]!='l' || b[4]!='-') return 0;
+	if(b[2]!='h' && b[2]!='z') return 0;
+	*pfoundpos = pos;
+	return 1;
+}
+
+static void lhasfx_find_payload(deark *c, struct lhasfx_context *d)
+{
+	if(d->sfx_container_is_larc) {
+		if(is_lharc_data_at(c, 594, &d->payload_offs)) {
+			goto done;
+		}
+	}
+	else {
+		if(is_lharc_data_at(c, 1260, &d->payload_offs) || // LHarc 1.12
+			is_lharc_data_at(c, 1263, &d->payload_offs) || // LHarc 1.13-1.14
+			is_lharc_data_at(c, 1290, &d->payload_offs)) // LHarc 1.00
+		{
+			goto done;
+		}
+	}
+
+done:
+	if(d->payload_offs==0) {
+		d->errflag = 1;
+		d->need_errmsg = 1;
+	}
+}
+
+static void de_run_lharc_sfx_com(deark *c, de_module_params *mparams)
+{
+	struct lhasfx_context *d = NULL;
+	int ret;
+
+	d = de_malloc(c, sizeof(struct lhasfx_context));
+
+	ret = looks_like_lharc_sfx_com(c, &d->sfx_container_is_larc);
+	if(!ret) {
+		d->errflag = 1;
+		d->need_errmsg = 1;
+		goto done;
+	}
+
+	de_declare_fmtf(c, "%s self-extracting archive (COM)",
+		(d->sfx_container_is_larc ? "LArc":"LHarc"));
+
+	lhasfx_find_payload(c, d);
+	if(d->errflag) goto done;
+
+	de_dbg(c, "payload found at: %"I64_FMT, d->payload_offs);
+
+	dbuf_create_file_from_slice(c->infile, d->payload_offs,
+		c->infile->len-d->payload_offs,
+		(d->sfx_container_is_larc ? "lzs" : "lha"), NULL, 0);
+
+done:
+	if(d->errflag && d->need_errmsg) {
+		de_err(c, "Not a known LHarc/LArc SFX format");
+	}
+	de_free(c, d);
+}
+
+static int de_identify_lharc_sfx_com(deark *c)
+{
+	int is_larc = 0;
+	int ret;
+
+	if(!de_input_file_has_ext(c, "com")) return 0;
+	ret = looks_like_lharc_sfx_com(c, &is_larc);
+	if(ret) return 70;
+	return 0;
+}
+
+void de_module_lharc_sfx_com(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "lharc_sfx_com";
+	mi->desc = "LHarc/LArc self-extracting archive (COM)";
+	mi->run_fn = de_run_lharc_sfx_com;
+	mi->identify_fn = de_identify_lharc_sfx_com;
+}
