@@ -9,20 +9,24 @@
 DE_DECLARE_MODULE(de_module_copy);
 DE_DECLARE_MODULE(de_module_null);
 DE_DECLARE_MODULE(de_module_split);
+DE_DECLARE_MODULE(de_module_xor);
 DE_DECLARE_MODULE(de_module_plaintext);
 DE_DECLARE_MODULE(de_module_cp437);
 DE_DECLARE_MODULE(de_module_crc);
 DE_DECLARE_MODULE(de_module_hexdump);
 DE_DECLARE_MODULE(de_module_bytefreq);
+DE_DECLARE_MODULE(de_module_deflate);
 DE_DECLARE_MODULE(de_module_zlib);
 DE_DECLARE_MODULE(de_module_winzle);
 DE_DECLARE_MODULE(de_module_mrw);
 DE_DECLARE_MODULE(de_module_vgafont);
+DE_DECLARE_MODULE(de_module_pcrfont);
 DE_DECLARE_MODULE(de_module_zbr);
 DE_DECLARE_MODULE(de_module_compress);
 DE_DECLARE_MODULE(de_module_hpi);
-DE_DECLARE_MODULE(de_module_dwc);
 DE_DECLARE_MODULE(de_module_dclimplode);
+DE_DECLARE_MODULE(de_module_lzss_oku);
+DE_DECLARE_MODULE(de_module_lzhuf);
 
 // **************************************************************************
 // "copy" module
@@ -144,41 +148,99 @@ void de_module_split(deark *c, struct deark_module_info *mi)
 	mi->help_fn = de_help_split;
 }
 
+// **************************************************************************
+// xor
+// Apply an XOR key to the file. Can be used to de-obfuscate some files.
+// **************************************************************************
+
+struct xorctx_struct {
+	dbuf *outf;
+	UI key_len;
+	UI key_pos;
+	u8 key[100];
+};
+
+static int xor_cbfn(struct de_bufferedreadctx *brctx, const u8 *buf,
+	i64 buf_len)
+{
+	i64 i;
+	struct xorctx_struct *xorctx = (struct xorctx_struct*)brctx->userdata;
+
+	for(i=0; i<buf_len; i++) {
+		dbuf_writebyte(xorctx->outf, buf[i] ^ xorctx->key[xorctx->key_pos]);
+		if(xorctx->key_len>1) {
+			xorctx->key_pos = (xorctx->key_pos+1) % xorctx->key_len;
+		}
+	}
+	return 1;
+}
+
+static void de_run_xor(deark *c, de_module_params *mparams)
+{
+	struct xorctx_struct *xorctx = NULL;
+	int badkey_flag = 0;
+	const char *opt_key;
+
+	xorctx = de_malloc(c, sizeof(struct xorctx_struct));
+	xorctx->key[0] = 0xff;
+	xorctx->key_len = 1;
+
+	opt_key = de_get_ext_option(c, "xor:key");
+	if(opt_key) {
+		UI key_strlen;
+		UI k;
+
+		key_strlen = (UI)de_strlen(opt_key);
+		if(key_strlen%2) { badkey_flag = 1; goto done; }
+		xorctx->key_len = key_strlen/2;
+		if(xorctx->key_len<1 || (size_t)xorctx->key_len>sizeof(xorctx->key)) {
+			badkey_flag = 1;
+			goto done;
+		}
+		for(k=0; k<xorctx->key_len; k++) {
+			u8 d1, d2;
+			int errflag1 = 0;
+			int errflag2 = 0;
+
+			d1 = de_decode_hex_digit(opt_key[k*2], &errflag1);
+			d2 = de_decode_hex_digit(opt_key[k*2+1], &errflag2);
+			if(errflag1 || errflag2) { badkey_flag = 1; goto done; }
+			xorctx->key[k] = (d1<<4) | d2;
+		}
+	}
+
+	xorctx->outf = dbuf_create_output_file(c, "bin", NULL, 0);
+	dbuf_enable_wbuffer(xorctx->outf);
+	dbuf_buffered_read(c->infile, 0, c->infile->len, xor_cbfn, (void*)xorctx);
+
+done:
+	if(badkey_flag) {
+		de_err(c, "Bad XOR key");
+	}
+	if(xorctx) {
+		dbuf_close(xorctx->outf);
+		de_free(c, xorctx);
+	}
+}
+
+static void de_help_xor(deark *c)
+{
+	de_msg(c, "-opt xor:key=<aabbcc...> : Hex bytes to use as XOR key");
+}
+
+void de_module_xor(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "xor";
+	mi->desc = "Invert bits, or XOR with a key";
+	mi->run_fn = de_run_xor;
+	mi->help_fn = de_help_xor;
+}
+
 // **************************************************************************
 // plaintext
 // Convert text files to UTF-8.
 // **************************************************************************
 
-struct plaintextctx_struct {
-	dbuf *outf;
-	de_ucstring *tmpstr;
-	struct de_encconv_state es;
-};
-
-static int plaintext_cbfn(struct de_bufferedreadctx *brctx, const u8 *buf,
-	i64 buf_len)
-{
-	struct plaintextctx_struct *ptctx = (struct plaintextctx_struct*)brctx->userdata;
-	UI conv_flags;
-
-	// There's no limit to how much data dbuf_buffered_read() could send us
-	// at once, so we won't try to put it all in a ucstring at once.
-	brctx->bytes_consumed = de_min_int(buf_len, 4096);
-
-	// For best results, ucstring_append_bytes_ex() needs to be told whether there will
-	// be any more bytes after this.
-	if(brctx->eof_flag && brctx->bytes_consumed==buf_len)
-		conv_flags = 0;
-	else
-		conv_flags = DE_CONVFLAG_PARTIAL_DATA;
-
-	ucstring_empty(ptctx->tmpstr);
-	ucstring_append_bytes_ex(ptctx->tmpstr, buf, brctx->bytes_consumed, conv_flags,
-		&ptctx->es);
-	ucstring_write_as_utf8(brctx->c, ptctx->tmpstr, ptctx->outf, 0);
-	return 1;
-}
-
 static de_encoding get_bom_enc(deark *c, UI *blen)
 {
 	u8 buf[3];
@@ -202,11 +264,11 @@ static de_encoding get_bom_enc(deark *c, UI *blen)
 
 static void de_run_plaintext(deark *c, de_module_params *mparams)
 {
-	struct plaintextctx_struct ptctx;
 	de_encoding input_encoding;
 	de_encoding enc_from_bom;
 	UI existing_bom_len = 0;
 	i64 dpos, dlen;
+	dbuf *outf = NULL;
 
 	enc_from_bom = get_bom_enc(c, &existing_bom_len);
 	input_encoding = de_get_input_encoding(c, NULL, DE_ENCODING_UNKNOWN);
@@ -226,17 +288,18 @@ static void de_run_plaintext(deark *c, de_module_params *mparams)
 	dpos = (i64)existing_bom_len;
 	dlen = c->infile->len - dpos;
 
-	de_encconv_init(&ptctx.es, DE_EXTENC_MAKE(input_encoding, DE_ENCSUBTYPE_HYBRID));
-	ptctx.tmpstr = ucstring_create(c);
-	ptctx.outf = dbuf_create_output_file(c, "txt", NULL, 0);
+	outf = dbuf_create_output_file(c, "txt", NULL, 0);
+	dbuf_enable_wbuffer(outf);
 
 	if(c->write_bom) {
-		dbuf_write_uchar_as_utf8(ptctx.outf, 0xfeff);
+		dbuf_write_uchar_as_utf8(outf, 0xfeff);
 	}
 
-	dbuf_buffered_read(c->infile, dpos, dlen, plaintext_cbfn, (void*)&ptctx);
-	dbuf_close(ptctx.outf);
-	ucstring_destroy(ptctx.tmpstr);
+	dbuf_copy_slice_convert_to_utf8(c->infile, dpos, dlen,
+		DE_EXTENC_MAKE(input_encoding, DE_ENCSUBTYPE_HYBRID),
+		outf, 0);
+
+	dbuf_close(outf);
 }
 
 void de_module_plaintext(deark *c, struct deark_module_info *mi)
@@ -292,6 +355,7 @@ static void de_run_cp437(deark *c, de_module_params *mparams)
 	struct cp437ctx_struct cp437ctx;
 
 	cp437ctx.outf = dbuf_create_output_file(c, "txt", NULL, 0);
+	dbuf_enable_wbuffer(cp437ctx.outf);
 	de_encconv_init(&cp437ctx.es, DE_ENCODING_CP437_G);
 	if(c->write_bom) {
 		dbuf_write_uchar_as_utf8(cp437ctx.outf, 0xfeff);
@@ -341,7 +405,7 @@ static void de_run_crc(deark *c, de_module_params *mparams)
 	de_zeromem(&crcctx, sizeof(struct crcctx_struct));
 	crcctx.crco_32ieee = de_crcobj_create(c, DE_CRCOBJ_CRC32_IEEE);
 	crcctx.crco_16arc = de_crcobj_create(c, DE_CRCOBJ_CRC16_ARC);
-	crcctx.crco_16ccitt = de_crcobj_create(c, DE_CRCOBJ_CRC16_CCITT);
+	crcctx.crco_16ccitt = de_crcobj_create(c, DE_CRCOBJ_CRC16_XMODEM);
 
 	dbuf_buffered_read(c->infile, 0, c->infile->len, crc_cbfn, (void*)&crcctx);
 
@@ -493,6 +557,61 @@ void de_module_bytefreq(deark *c, struct deark_module_info *mi)
 	mi->flags |= DE_MODFLAG_NOEXTRACT;
 }
 
+// **************************************************************************
+// Raw "Deflate"-compressed data
+// **************************************************************************
+
+static void run_deflate_internal(deark *c, UI flags)
+{
+	dbuf *outf = NULL;
+	struct de_dfilter_in_params dcmpri;
+	struct de_dfilter_out_params dcmpro;
+	struct de_dfilter_results dres;
+	struct de_deflate_params deflparams;
+
+	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
+	de_zeromem(&deflparams, sizeof(struct de_deflate_params));
+	deflparams.flags = flags;
+
+	outf = dbuf_create_output_file(c, "unc", NULL, 0);
+	dbuf_enable_wbuffer(outf);
+	dcmpri.f = c->infile;
+	dcmpri.pos = 0;
+	dcmpri.len = c->infile->len;
+	dcmpro.f = outf;
+	fmtutil_decompress_deflate_ex(c, &dcmpri, &dcmpro, &dres, &deflparams);
+	dbuf_flush(outf);
+	if(dres.errcode) {
+		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
+		goto done;
+	}
+	if(dres.bytes_consumed_valid) {
+		de_dbg(c, "decompressed %"I64_FMT" bytes to %"I64_FMT, dres.bytes_consumed,
+			outf->len);
+	}
+
+done:
+	dbuf_close(outf);
+}
+
+static void de_run_deflate(deark *c, de_module_params *mparams)
+{
+	UI flags = 0;
+
+	if(de_get_ext_option_bool(c, "deflate:deflate64", 0)) {
+		flags |= DE_DEFLATEFLAG_DEFLATE64;
+	}
+
+	run_deflate_internal(c, flags);
+}
+
+void de_module_deflate(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "deflate";
+	mi->desc = "Raw Deflate compressed data";
+	mi->run_fn = de_run_deflate;
+}
+
 // **************************************************************************
 // zlib module
 //
@@ -501,11 +620,7 @@ void de_module_bytefreq(deark *c, struct deark_module_info *mi)
 
 static void de_run_zlib(deark *c, de_module_params *mparams)
 {
-	dbuf *f = NULL;
-
-	f = dbuf_create_output_file(c, "unc", NULL, 0);
-	fmtutil_decompress_deflate(c->infile, 0, c->infile->len, f, 0, NULL, DE_DEFLATEFLAG_ISZLIB);
-	dbuf_close(f);
+	run_deflate_internal(c, DE_DEFLATEFLAG_ISZLIB);
 }
 
 static int de_identify_zlib(deark *c)
@@ -628,29 +743,29 @@ void de_module_mrw(deark *c, struct deark_module_info *mi)
 }
 
 // **************************************************************************
-// VGA font (intended for development/debugging use)
+// 8xN "VGA" font (intended for development/debugging use)
 // **************************************************************************
 
-static void de_run_vgafont(deark *c, de_module_params *mparams)
+static void do_vgafont_internal(deark *c, dbuf *inf)
 {
 	u8 *fontdata = NULL;
 	struct de_bitmap_font *font = NULL;
 	i64 i;
 	i64 height;
+	de_encoding encoding_req;
+	de_encoding encoding_to_use;
+	struct de_encconv_state es;
 
-	if(c->infile->len==16*256) {
-		height = 16;
-	}
-	else if(c->infile->len==14*256) {
-		height = 14;
-	}
-	else {
+#define VGAFONT_MINH 3
+#define VGAFONT_MAXH 20
+	height = inf->len / 256;
+	if((inf->len % 256) || height<VGAFONT_MINH || height>VGAFONT_MAXH) {
 		de_err(c, "Bad file size");
 		goto done;
 	}
 
 	fontdata = de_malloc(c, height*256);
-	de_read(fontdata, 0, height*256);
+	dbuf_read(inf, fontdata, 0, height*256);
 
 	if(de_get_ext_option(c, "vgafont:c")) {
 		dbuf *ff;
@@ -665,17 +780,28 @@ static void de_run_vgafont(deark *c, de_module_params *mparams)
 		goto done;
 	}
 
+	encoding_req = de_get_input_encoding(c, NULL, DE_ENCODING_UNKNOWN);
+	if(encoding_req!=DE_ENCODING_UNKNOWN)
+		encoding_to_use = encoding_req;
+	else
+		encoding_to_use = DE_ENCODING_CP437;
+
 	font = de_create_bitmap_font(c);
 	font->num_chars = 256;
 	font->has_nonunicode_codepoints = 1;
-	font->has_unicode_codepoints = 0;
-	font->prefer_unicode = 0;
+	font->has_unicode_codepoints = 1;
+	font->prefer_unicode = (encoding_req!=DE_ENCODING_UNKNOWN);
 	font->nominal_width = 8;
 	font->nominal_height = (int)height;
 	font->char_array = de_mallocarray(c, font->num_chars, sizeof(struct de_bitmap_font_char));
 
+	de_encconv_init(&es, encoding_to_use);
+
 	for(i=0; i<font->num_chars; i++) {
 		font->char_array[i].codepoint_nonunicode = (i32)i;
+		if(font->has_unicode_codepoints) {
+			font->char_array[i].codepoint_unicode = de_char_to_unicode_ex((i32)i, &es);
+		}
 		font->char_array[i].width = font->nominal_width;
 		font->char_array[i].height = font->nominal_height;
 		font->char_array[i].rowspan = 1;
@@ -692,6 +818,11 @@ done:
 	de_free(c, fontdata);
 }
 
+static void de_run_vgafont(deark *c, de_module_params *mparams)
+{
+	do_vgafont_internal(c, c->infile);
+}
+
 static void de_help_vgafont(deark *c)
 {
 	de_msg(c, "-opt vgafont:c : Emit C code");
@@ -700,10 +831,64 @@ static void de_help_vgafont(deark *c)
 void de_module_vgafont(deark *c, struct deark_module_info *mi)
 {
 	mi->id = "vgafont";
-	mi->desc = "Raw 8x16 or 8x14 VGA font";
+	mi->desc = "Raw 8xN bitmap font";
 	mi->run_fn = de_run_vgafont;
 	mi->help_fn = de_help_vgafont;
-	mi->flags |= DE_MODFLAG_HIDDEN;
+}
+
+// **************************************************************************
+// PCR font (OPTICKS)
+// **************************************************************************
+
+static void de_run_pcrfont(deark *c, de_module_params *mparams)
+{
+	u8 hdr[11];
+	i64 h;
+	int need_errmsg = 0;
+	dbuf *inf = NULL;
+
+	de_read(hdr, 0, 11);
+	// I assume either hdr[7] or hdr[10] is the high byte of the font data size,
+	// but I don't know which.
+	if(hdr[6]!=0x1 || hdr[7]!=hdr[10] || hdr[8]!=0 || hdr[9]!=0) {
+		need_errmsg = 1;
+		goto done;
+	}
+	h = (i64)hdr[7];
+	de_dbg(c, "height: %d", (int)h);
+	if(h<VGAFONT_MINH || h>VGAFONT_MAXH || c->infile->len < 11+h*256) {
+		need_errmsg = 1;
+		goto done;
+	}
+
+	inf = dbuf_open_input_subfile(c->infile, 11, h*256);
+	do_vgafont_internal(c, inf);
+
+done:
+	dbuf_close(inf);
+	if(need_errmsg) {
+		de_err(c, "Unsupported type of PCR font");
+	}
+}
+
+static int de_identify_pcrfont(deark *c)
+{
+	u8 h;
+
+	if(dbuf_memcmp(c->infile, 0, "KPG", 3)) return 0;
+	if(de_getbyte(5)!=0x20) return 0;
+	h = de_getbyte(7);
+	if(h<6 || h>16) return 0;
+	if(de_getbyte(10)!=h) return 0;
+	return 80;
+}
+
+void de_module_pcrfont(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "pcrfont";
+	mi->desc = "PCR font";
+	mi->run_fn = de_run_pcrfont;
+	mi->identify_fn = de_identify_pcrfont;
 }
 
 // **************************************************************************
@@ -841,96 +1026,6 @@ void de_module_hpi(deark *c, struct deark_module_info *mi)
 	mi->identify_fn = de_identify_hpi;
 }
 
-// **************************************************************************
-// DWC archive
-// **************************************************************************
-
-struct dwc_member_data {
-	de_ucstring *fn;
-	i64 unclen, cmprlen;
-	i64 cmprdatapos;
-	u8 cmpr_meth;
-};
-
-static void do_dwc_member(deark *c, i64 pos1)
-{
-	i64 pos = pos1;
-	struct dwc_member_data *md = NULL;
-
-	md = de_malloc(c, sizeof(struct dwc_member_data));
-	de_dbg(c, "member header at %"I64_FMT, pos1);
-	de_dbg_indent(c, 1);
-	md->fn = ucstring_create(c);
-	dbuf_read_to_ucstring(c->infile, pos, 12, md->fn, DE_CONVFLAG_STOP_AT_NUL,
-		DE_ENCODING_CP437_G);
-	de_dbg(c, "filename: \"%s\"", ucstring_getpsz_d(md->fn));
-	pos += 13;
-	md->unclen = de_getu32le_p(&pos);
-	de_dbg(c, "unc. size: %"I64_FMT, md->unclen);
-	pos += 4; // time/date
-	md->cmprlen = de_getu32le_p(&pos);
-	de_dbg(c, "cmpr. size: %"I64_FMT, md->cmprlen);
-	md->cmprdatapos = de_getu32le_p(&pos);
-	de_dbg(c, "cmpr. data pos: %"I64_FMT, md->cmprdatapos);
-	md->cmpr_meth = de_getbyte_p(&pos);
-	de_dbg(c, "cmpr. method: %d", (int)md->cmpr_meth);
-	// pos += 2; // ?
-	// pos += 2; // CRC
-	de_dbg_indent(c, -1);
-	if(md) {
-		ucstring_destroy(md->fn);
-		de_free(c, md);
-	}
-}
-
-static void de_run_dwc(deark *c, de_module_params *mparams)
-{
-	i64 trailer_pos;
-	i64 nmembers;
-	i64 fhsize; // size of each file header
-	i64 pos;
-	i64 i;
-
-	de_info(c, "Note: DWC files can be parsed, but no files can be extracted from them.");
-	trailer_pos = c->infile->len - 27;
-	pos = trailer_pos;
-	de_dbg(c, "trailer at %"I64_FMT, trailer_pos);
-	de_dbg_indent(c, 1);
-	pos += 2; // trailer length (27)
-	fhsize = de_getu16le_p(&pos);
-	pos += 16;
-	nmembers = de_getu16le_p(&pos);
-	de_dbg(c, "number of member files: %d", (int)nmembers);
-	de_dbg_indent(c, -1);
-
-	pos = trailer_pos - fhsize*nmembers;
-	for(i=0; i<nmembers; i++) {
-		if(pos<0 || pos>(trailer_pos-fhsize)) break;
-		do_dwc_member(c, pos);
-		pos += fhsize;
-	}
-}
-
-static int de_identify_dwc(deark *c)
-{
-	if(!de_input_file_has_ext(c, "dwc")) return 0;
-	if(dbuf_memcmp(c->infile, c->infile->len-3, (const u8*)"DWC", 3)) {
-		return 0;
-	}
-	if(de_getu16le(c->infile->len-27) != 27) {
-		return 0;
-	}
-	return 70;
-}
-
-void de_module_dwc(deark *c, struct deark_module_info *mi)
-{
-	mi->id = "dwc";
-	mi->desc = "DWC compressed archive";
-	mi->run_fn = de_run_dwc;
-	mi->identify_fn = de_identify_dwc;
-}
-
 // **************************************************************************
 // PKWARE DCL Implode compressed file
 // **************************************************************************
@@ -943,6 +1038,7 @@ static void de_run_dclimplode(deark *c, de_module_params *mparams)
 	struct de_dfilter_results dres;
 
 	outf = dbuf_create_output_file(c, "unc", NULL, 0);
+	dbuf_enable_wbuffer(outf);
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
 	dcmpri.f = c->infile;
 	dcmpri.pos = 0;
@@ -950,6 +1046,7 @@ static void de_run_dclimplode(deark *c, de_module_params *mparams)
 	dcmpro.f = outf;
 
 	fmtutil_dclimplode_codectype1(c, &dcmpri, &dcmpro, &dres, NULL);
+	dbuf_flush(outf);
 	if(dres.errcode) {
 		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
 	}
@@ -960,6 +1057,8 @@ static void de_run_dclimplode(deark *c, de_module_params *mparams)
 static int de_identify_dclimplode(deark *c)
 {
 	u8 b0, b1;
+	int i;
+	u32 x;
 
 	if(c->infile->len<5) return 0;
 	b0 = de_getbyte(0);
@@ -970,8 +1069,7 @@ static int de_identify_dclimplode(deark *c)
 	// Look for the end-of-data code in the last 2 or 3 bytes.
 	// Assumes the last byte is padded with '0' bits, and there are
 	// no extraneous bytes after that.
-	u32 x = (u32)de_getu32le(c->infile->len-4);
-	int i;
+	x = (u32)de_getu32le(c->infile->len-4);
 	for(i=0; i<8; i++) {
 		if((x & 0xfffffc00U)==0x01fe0000U) {
 			if(b0==0 && b1==6) return 40;
@@ -990,3 +1088,85 @@ void de_module_dclimplode(deark *c, struct deark_module_info *mi)
 	mi->run_fn = de_run_dclimplode;
 	mi->identify_fn = de_identify_dclimplode;
 }
+
+// **************************************************************************
+// LZSS (Haruhiko Okumura) compressed file
+// **************************************************************************
+
+static void de_run_lzss_oku(deark *c, de_module_params *mparams)
+{
+	dbuf *outf = NULL;
+	struct de_dfilter_in_params dcmpri;
+	struct de_dfilter_out_params dcmpro;
+	struct de_dfilter_results dres;
+
+	outf = dbuf_create_output_file(c, "unc", NULL, 0);
+	dbuf_enable_wbuffer(outf);
+	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
+	dcmpri.f = c->infile;
+	dcmpri.pos = 0;
+	dcmpri.len = c->infile->len;
+	dcmpro.f = outf;
+
+	fmtutil_decompress_lzss1(c, &dcmpri, &dcmpro, &dres, 0x0);
+	dbuf_flush(outf);
+	if(dres.errcode) {
+		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
+	}
+
+	dbuf_close(outf);
+}
+
+void de_module_lzss_oku(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "lzss_oku";
+	mi->desc = "LZSS.C by Haruhiko Okumura";
+	mi->run_fn = de_run_lzss_oku;
+}
+
+// **************************************************************************
+// LZHUF (Haruyasu Yoshizaki) compressed file
+// **************************************************************************
+
+static void de_run_lzhuf(deark *c, de_module_params *mparams)
+{
+	i64 unc_filesize;
+	dbuf *outf = NULL;
+	struct de_dfilter_in_params dcmpri;
+	struct de_dfilter_out_params dcmpro;
+	struct de_dfilter_results dres;
+
+	// We're assuming the size field is 4 bytes, little-endian. (But it could
+	// be platform-specific.)
+#define LZHUF_HDRSIZE 4
+#define LZHUF_IS_LE   1
+
+	if(c->infile->len<LZHUF_HDRSIZE) goto done;
+	unc_filesize = dbuf_getint_ext(c->infile, 0, LZHUF_HDRSIZE, LZHUF_IS_LE, 0);
+	de_dbg(c, "orig filesize: %"I64_FMT, unc_filesize);
+
+	outf = dbuf_create_output_file(c, "unc", NULL, 0);
+	dbuf_enable_wbuffer(outf);
+	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
+	dcmpri.f = c->infile;
+	dcmpri.pos = LZHUF_HDRSIZE;
+	dcmpri.len = c->infile->len-LZHUF_HDRSIZE;
+	dcmpro.f = outf;
+	dcmpro.len_known = 1;
+	dcmpro.expected_len = unc_filesize;
+
+	fmtutil_lh1_codectype1(c, &dcmpri, &dcmpro, &dres, NULL);
+	dbuf_flush(outf);
+	if(dres.errcode) {
+		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
+	}
+done:
+	dbuf_close(outf);
+}
+
+void de_module_lzhuf(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "lzhuf";
+	mi->desc = "LZHUF compressed file";
+	mi->run_fn = de_run_lzhuf;
+}
