@@ -361,7 +361,7 @@ static void mslzh_read_huffman_tree(struct mslzh_context *lzhctx, UI idx)
 			(UI)htr->symlengths[i]);
 	}
 
-	if(!fmtutil_huffman_make_canonical_code(c, htr->fmtuht->bk, htr->fmtuht->builder, 0)) {
+	if(!fmtutil_huffman_make_canonical_code(c, htr->fmtuht->bk, htr->fmtuht->builder, 0, NULL)) {
 		de_dfilter_set_errorf(c, lzhctx->dres, lzhctx->modname, "Failed to construct Huffman tree");
 		mslzh_set_errorflag(lzhctx);
 		goto done;
@@ -600,6 +600,7 @@ static void do_decompress_MSZIP(deark *c, struct de_dfilter_in_params *dcmpri1,
 		outlen_before = dcmpro->f->len;
 
 		fmtutil_deflate_codectype1(c, &dcmpri2, dcmpro, dres, (void*)&inflparams);
+		dbuf_flush(dcmpro->f);
 		if(dres->errcode) goto done;
 
 		pos += blk_dlen;
@@ -640,7 +641,7 @@ static void do_decompress(deark *c, lctx *d, dbuf *outf)
 		do_decompress_XOR(c, &dcmpri, &dcmpro, &dres);
 		break;
 	case CMPR_SZDD:
-		fmtutil_decompress_szdd(c, &dcmpri, &dcmpro, &dres, 0);
+		fmtutil_decompress_lzss1(c, &dcmpri, &dcmpro, &dres, 0x1);
 		break;
 	case CMPR_LZHUFF:
 		do_decompress_LZHUFF(c, &dcmpri, &dcmpro, &dres);
@@ -649,6 +650,7 @@ static void do_decompress(deark *c, lctx *d, dbuf *outf)
 		do_decompress_MSZIP(c, &dcmpri, &dcmpro, &dres);
 		break;
 	}
+	dbuf_flush(dcmpro.f);
 
 	if(dres.errcode) {
 		de_err(c, "%s", de_dfilter_get_errmsg(c, &dres));
@@ -692,6 +694,7 @@ static void do_extract_file(deark *c, lctx *d)
 		de_finfo_set_name_from_sz(c, fi, "bin", 0, DE_ENCODING_LATIN1);
 	}
 	outf = dbuf_create_output_file(c, NULL, fi, 0);
+	dbuf_enable_wbuffer(outf);
 	do_decompress(c, d, outf);
 	de_dbg_indent(c, -1);
 
