@@ -33,7 +33,8 @@ typedef struct localctx_struct {
 	u8 palette_info;
 	u8 reserved1;
 	i64 width, height;
-	int is_mswordscr;
+	u8 is_mswordscr;
+	u8 is_pcxsfx;
 	int has_vga_pal;
 	int has_transparency;
 
@@ -120,6 +121,7 @@ static void do_decode_resolution(deark *c, lctx *d, i64 hres, i64 vres)
 
 static int do_read_header(deark *c, lctx *d)
 {
+	u8 initialbyte;
 	int retval = 0;
 	i64 hres, vres;
 	const char *imgtypename = "";
@@ -127,7 +129,25 @@ static int do_read_header(deark *c, lctx *d)
 	de_dbg(c, "header at %d", 0);
 	de_dbg_indent(c, 1);
 
+	initialbyte = de_getbyte(0);
 	d->version = de_getbyte(1);
+	if(!d->is_mswordscr) {
+		if(initialbyte==0xeb && d->version==0x0e) {
+			d->is_pcxsfx = 1;
+			d->version = 5;
+		}
+	}
+
+	if(d->is_mswordscr) {
+		de_declare_fmt(c, "Word for DOS screen capture");
+	}
+	else if(d->is_pcxsfx) {
+		de_declare_fmt(c, "VGAPaint 386 PCX-SFX");
+	}
+	else {
+		de_declare_fmt(c, "PCX");
+	}
+
 	d->encoding = de_getbyte(2);
 	d->bits = (i64)de_getbyte(3); // Bits per pixel per plane
 	d->margin_L = de_getu16le(4);
@@ -378,6 +398,7 @@ static void do_palette_stuff(deark *c, lctx *d)
 		u8 p0, p3;
 		unsigned int bgcolor;
 		unsigned int fgpal;
+		int pal_subid;
 
 		de_warn(c, "4-color PCX images might not be supported correctly");
 
@@ -388,29 +409,30 @@ static void do_palette_stuff(deark *c, lctx *d)
 		de_dbg(c, "using a CGA palette: palette #%d, bkgd color %d", (int)fgpal, (int)bgcolor);
 
 		// Set first pal entry to background color
-		d->pal[0] = de_palette_pc16(bgcolor);
+		d->pal[0] = de_get_std_palette_entry(DE_PALID_PC16, 0, (int)bgcolor);
 
 		// TODO: These palettes are quite possibly incorrect. I can't find good
 		// information about them.
 		switch(fgpal) {
-		case 0: case 2: // C=0 P=? I=0
-			d->pal[1]=0x00aaaa; d->pal[2]=0xaa0000; d->pal[3]=0xaaaaaa; break;
-		case 1: case 3: // C=0 P=? I=1
-			d->pal[1]=0x55ffff; d->pal[2]=0xff5555; d->pal[3]=0xffffff; break;
-		case 4: // C=1 P=0 I=0
-			d->pal[1]=0x00aa00; d->pal[2]=0xaa0000; d->pal[3]=0xaa5500; break;
-		case 5: // C=1 P=0 I=1
-			d->pal[1]=0x55ff55; d->pal[2]=0xff5555; d->pal[3]=0xffff55; break;
-		case 6: // C=1 P=1 I=0
-			d->pal[1]=0x00aaaa; d->pal[2]=0xaa00aa; d->pal[3]=0xaaaaaa; break;
-		case 7: // C=1 P=1 I=1
-			d->pal[1]=0x55ffff; d->pal[2]=0xff55ff; d->pal[3]=0xffffff; break;
+		case 1: case 3:
+			pal_subid = 5; break; // C=0 P=? I=1
+		case 4:
+			pal_subid = 1; break; // C=1 P=0 I=0
+		case 5:
+			pal_subid = 4; break; // C=1 P=0 I=1
+		case 6:
+			pal_subid = 0; break; // C=1 P=1 I=0
+		case 7:
+			pal_subid = 3; break; // C=1 P=1 I=1
+		default: // 0, 2
+			pal_subid = 2; break; // C=0 P=? I=0
 		}
+		de_copy_std_palette(DE_PALID_CGA, pal_subid, 1, 3, &d->pal[1], 3, 0);
 		return;
 	}
 
 	if(d->ncolors>16 && d->ncolors<=256) {
-		de_warn(c, "No suitable palette found");
+		de_warn(c, "%u-color image format with 16-color palette", (UI)d->ncolors);
 	}
 
 	de_dbg(c, "using 16-color palette from header");
@@ -433,6 +455,7 @@ static int do_uncompress(deark *c, lctx *d)
 
 	expected_bytes = d->rowspan * d->height;
 	d->unc_pixels = dbuf_create_membuf(c, expected_bytes, 0);
+	dbuf_enable_wbuffer(d->unc_pixels);
 
 	endpos = c->infile->len;
 	if(d->has_vga_pal) {
@@ -460,6 +483,7 @@ static int do_uncompress(deark *c, lctx *d)
 		}
 	}
 
+	dbuf_flush(d->unc_pixels);
 	if(d->unc_pixels->len < expected_bytes) {
 		de_warn(c, "Expected %d bytes of image data, only found %d",
 			(int)expected_bytes, (int)d->unc_pixels->len);
@@ -595,6 +619,7 @@ static void de_run_pcx_internal(deark *c, lctx *d, de_module_params *mparams)
 			goto done;
 		}
 	}
+	dbuf_flush(d->unc_pixels);
 
 	do_bitmap(c, d);
 
@@ -625,9 +650,18 @@ static int de_identify_pcx(deark *c)
 	{
 		if(de_input_file_has_ext(c, "pcx"))
 			return 100;
-
 		return 16;
 	}
+
+	// VGAPaint 386 PCX SFX
+	if(buf[0]==0xeb && buf[1]==0x0e && buf[2]==1 && buf[3]==8 &&
+		(de_getbyte(16)==0xe8))
+	{
+		if(de_input_file_has_ext(c, "pcx"))
+			return 80;
+		return 8;
+	}
+
 	return 0;
 }
 
