@@ -107,8 +107,13 @@ static const char *get_member_name_for_msg(deark *c, lctx *d, struct member_data
 
 static void do_extract_comment(deark *c, lctx *d, i64 pos, i64 len, int is_main)
 {
-	dbuf_create_file_from_slice(c->infile, pos, len, "comment.txt",
-		NULL, DE_CREATEFLAG_IS_AUX);
+	dbuf *outf = NULL;
+
+	outf = dbuf_create_output_file(c, "comment.txt", NULL, DE_CREATEFLAG_IS_AUX);
+	dbuf_copy_slice_convert_to_utf8(c->infile, pos, len,
+		DE_EXTENC_MAKE(d->input_encoding, DE_ENCSUBTYPE_HYBRID),
+		outf, 0x2|0x4);
+	dbuf_close(outf);
 }
 
 static void do_dbg_comment(deark *c, lctx *d, i64 pos, i64 len, const char *name,
@@ -119,7 +124,7 @@ static void do_dbg_comment(deark *c, lctx *d, i64 pos, i64 len, const char *name
 	if(c->debug_level<1) return;
 	s = ucstring_create(c);
 	dbuf_read_to_ucstring_n(c->infile, pos, len, DE_DBG_MAX_STRLEN, s,
-		0, d->input_encoding);
+		0, DE_EXTENC_MAKE(d->input_encoding, DE_ENCSUBTYPE_HYBRID));
 	de_dbg(c, "%s: \"%s\"", name, ucstring_getpsz_d(s));
 	ucstring_destroy(s);
 }
@@ -544,13 +549,6 @@ static void decompress_lzh(deark *c, struct de_dfilter_in_params *dcmpri,
 	fmtutil_decompress_lh5x(c, dcmpri, dcmpro, dres, &lzhparams);
 }
 
-static void our_writelistener_cb(dbuf *f, void *userdata, const u8 *buf, i64 buf_len)
-{
-	struct de_crcobj *crco = (struct de_crcobj *)userdata;
-
-	de_crcobj_addbuf(crco, buf, buf_len);
-}
-
 // Process a single member file (or "trailer" record).
 // If there are more members after this, sets *next_member_hdr_pos to nonzero.
 static void do_member(deark *c, lctx *d, i64 pos1, i64 *next_member_hdr_pos)
@@ -616,7 +614,8 @@ static void do_member(deark *c, lctx *d, i64 pos1, i64 *next_member_hdr_pos)
 		ext = "bin";
 	}
 	outf = dbuf_create_output_file(c, ext, md->fi, 0);
-	dbuf_set_writelistener(outf, our_writelistener_cb, (void*)d->crco);
+	dbuf_enable_wbuffer(outf);
+	dbuf_set_writelistener(outf, de_writelistener_for_crc, (void*)d->crco);
 	de_crcobj_reset(d->crco);
 
 	dcmpri.f = c->infile;
@@ -641,6 +640,7 @@ static void do_member(deark *c, lctx *d, i64 pos1, i64 *next_member_hdr_pos)
 	default:
 		goto done; // Should be impossible
 	}
+	dbuf_flush(dcmpro.f);
 	de_dbg_indent(c, -1);
 
 	md->crc_calculated = de_crcobj_getval(d->crco);
@@ -812,8 +812,9 @@ static void de_run_zoo_filter(deark *c, de_module_params *mparams)
 	de_dbg(c, "crc (reported): 0x%04x", (UI)crc_reported);
 
 	outf = dbuf_create_output_file(c, "bin", NULL, 0);
+	dbuf_enable_wbuffer(outf);
 	crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_ARC);
-	dbuf_set_writelistener(outf, our_writelistener_cb, (void*)crco);
+	dbuf_set_writelistener(outf, de_writelistener_for_crc, (void*)crco);
 
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
 	dcmpri.f = c->infile;
@@ -829,6 +830,7 @@ static void de_run_zoo_filter(deark *c, de_module_params *mparams)
 	else {
 		decompress_lzd(c, &dcmpri, &dcmpro, &dres);
 	}
+	dbuf_flush(dcmpro.f);
 
 	if(dres.errcode) {
 		de_err(c, "%s", de_dfilter_get_errmsg(c, &dres));
