@@ -59,6 +59,7 @@ struct de_bmpinfo {
 #define DE_BMPINFO_ICO_FORMAT     0x2
 #define DE_BMPINFO_HAS_HOTSPOT    0x4
 #define DE_BMPINFO_CMPR_IS_4CC    0x8
+#define DE_BMPINFO_NOERR          0x10
 
 void fmtutil_get_bmp_compression_name(u32 code, char *s, size_t s_len,
 	int is_os2v2);
@@ -120,15 +121,24 @@ int fmtutil_decompress_packbits(dbuf *f, i64 pos1, i64 len,
 void fmtutil_decompress_rle90_ex(deark *c, struct de_dfilter_in_params *dcmpri,
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
 	unsigned int flags);
-void fmtutil_decompress_szdd(deark *c, struct de_dfilter_in_params *dcmpri,
+
+struct de_lzss1_params {
+	UI flags;
+};
+void fmtutil_decompress_lzss1(deark *c, struct de_dfilter_in_params *dcmpri,
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
-	unsigned int flags);
+	UI flags);
+void fmtutil_lzss1_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params);
+
 void fmtutil_hlp_lz77_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
 	void *codec_private_params);
 void fmtutil_huff_squeeze_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
 	void *codec_private_params);
+int fmtutil_decompress_exepack_reloc_tbl(deark *c, i64 pos1, i64 endpos, dbuf *outf);
 
 struct de_dfilter_ctx;
 typedef void (*dfilter_codec_type)(struct de_dfilter_ctx *dfctx, void *codec_private_params);
@@ -158,7 +168,8 @@ enum de_lzwfmt_enum {
 	DE_LZWFMT_ZIPSHRINK,
 	DE_LZWFMT_ZOOLZD,
 	DE_LZWFMT_TIFF,
-	DE_LZWFMT_ARC5
+	DE_LZWFMT_ARC5,
+	DE_LZWFMT_DWC
 };
 
 struct de_lzw_params {
@@ -539,6 +550,7 @@ struct de_advfile {
 	de_ucstring *filename;
 	unsigned int snflags; // flags for de_finfo_set_name*
 	unsigned int createflags;
+	u8 enable_wbuffer;
 	u8 original_filename_flag;
 	u8 no_applesingle;
 	u8 no_appledouble;
@@ -643,7 +655,7 @@ int fmtutil_huffman_record_a_code_length(deark *c, struct fmtutil_huffman_code_b
 #define FMTUTIL_MCTFLAG_LEFT_ALIGN_BRANCHES   0x1
 #define FMTUTIL_MCTFLAG_LAST_CODE_FIRST       0x2 // Pretend codes were added in the reverse order
 int fmtutil_huffman_make_canonical_code(deark *c, struct fmtutil_huffman_codebook *bk,
-	struct fmtutil_huffman_code_builder *builder, UI flags);
+	struct fmtutil_huffman_code_builder *builder, UI flags, const char *title);
 
 typedef void (*fmtutil_lz77buffer_cb_type)(struct de_lz77buffer *rb, u8 n);
 
@@ -661,3 +673,46 @@ struct de_lz77buffer {
  void de_lz77buffer_set_curpos(struct de_lz77buffer *rb, UI newpos);
  void de_lz77buffer_add_literal_byte(struct de_lz77buffer *rb, u8 b);
  void de_lz77buffer_copy_from_hist(struct de_lz77buffer *rb, UI startpos, UI count);
+
+ struct fmtutil_exe_info {
+	dbuf *f;
+	i64 num_relocs;
+	i64 regSS;
+	i64 regSP;
+	i64 regCS;
+	i64 regIP;
+	i64 reloc_table_pos;
+	i64 start_of_dos_code;
+	i64 entry_point;
+	i64 end_of_dos_code;
+	u64 entrypoint_crcs;
+};
+void fmtutil_collect_exe_info(deark *c, dbuf *f, struct fmtutil_exe_info *ei);
+
+#define DE_SPECIALEXEFMT_LZEXE     1
+#define DE_SPECIALEXEFMT_PKLITE    2
+#define DE_SPECIALEXEFMT_EXEPACK   3
+#define DE_SPECIALEXEFMT_DIET      4
+#define DE_SPECIALEXEFMT_TINYPROG  5
+#define DE_SPECIALEXEFMT_EXECOMP   99 // Misc. unsupported executable compression
+#define DE_SPECIALEXEFMT_SFX       100 // Generic extractable self-extracting archive
+#define DE_SPECIALEXEFMT_ZIPSFX    101
+
+ struct fmtutil_specialexe_detection_data {
+	u8 restrict_to_fmt; // DE_SPECIALEXEFMT_*; 0 = any
+	u8 detected_fmt; // DE_SPECIALEXEFMT_*; 0 = unknown
+	u8 detected_subfmt;
+	u8 payload_valid;
+	u8 zip_eocd_looked_for;
+	u8 zip_eocd_found;
+	i64 payload_pos;
+	i64 payload_len;
+	i64 zip_eocd_pos;
+	const char *payload_file_ext;
+	const char *modname; // Non-NULL if we think we can decompress
+	char detected_fmt_name[40];
+};
+void fmtutil_detect_execomp(deark *c, struct fmtutil_exe_info *ei,
+	struct fmtutil_specialexe_detection_data *edd);
+void fmtutil_detect_exesfx(deark *c, struct fmtutil_exe_info *ei,
+	struct fmtutil_specialexe_detection_data *edd);
