@@ -69,6 +69,7 @@ struct dbuf_struct;
 typedef struct dbuf_struct dbuf;
 struct de_finfo_struct;
 typedef struct de_finfo_struct de_finfo;
+struct de_crcobj;
 
 struct de_module_params_struct;
 typedef struct de_module_params_struct de_module_params;
@@ -166,16 +167,22 @@ struct dbuf_struct {
 	int file_pos_known;
 	i64 file_pos;
 
+	i64 wbuffer_bytes_used;
+	u8 *wbuffer;
+
 	struct dbuf_struct *parent_dbuf; // used for DBUF_TYPE_DBUF
 	i64 offset_into_parent_dbuf; // used for DBUF_TYPE_DBUF
 
 	u8 write_memfile_to_zip_archive;
 	u8 writing_to_tar_archive;
+	int file_id; // if managed
 	char *name; // used for DBUF_TYPE_OFILE (utf-8)
 
 	i64 membuf_alloc;
 	u8 *membuf_buf;
 
+	struct de_crcobj *crco_for_oinfo;
+
 	void *userdata_for_writelistener;
 	de_writelistener_cb_type writelistener_cb;
 	void *userdata_for_customread;
@@ -183,15 +190,15 @@ struct dbuf_struct {
 	void *userdata_for_customwrite;
 	de_dbufcustomwrite_type customwrite_fn; // used for DBUF_TYPE_CUSTOM
 
-#define DE_CACHE_POLICY_NONE    0
-#define DE_CACHE_POLICY_ENABLED 1
-	int cache_policy;
-	i64 cache_bytes_used;
-	u8 *cache; // first 'cache_bytes_used' bytes of the file
+#define DE_RCACHE_POLICY_NONE    0
+#define DE_RCACHE_POLICY_ENABLED 1
+	int rcache_policy;
+	i64 rcache_bytes_used;
+	u8 *rcache; // first 'cache_bytes_used' bytes of the file
 
 	// cache2 is a simple 1-byte cache, mainly to speed up de_get_bits_symbol().
-	i64 cache2_pos;
-	u8 cache2;
+	i64 rcache2_pos;
+	u8 rcache2;
 
 	// Things copied from the de_finfo object at file creation
 	de_finfo *fi_copy;
@@ -289,6 +296,7 @@ struct de_module_in_params {
 	i64 offset_in_parent;
 	dbuf *parent_dbuf;
 	de_finfo *fi;
+	void *obj1;
 };
 
 struct de_module_out_params {
@@ -379,6 +387,7 @@ struct deark_struct {
 	int extract_level;
 	u8 list_mode;
 	u8 list_mode_include_file_id;
+	u8 enable_oinfo;
 	int first_output_file; // first file = 0
 	int max_output_files;
 	u8 user_set_max_output_files;
@@ -418,6 +427,8 @@ struct deark_struct {
 	u8 deflate_decoder_id;
 	u8 tmpflag1;
 	u8 tmpflag2;
+	u8 enable_wbuffer_test;
+	u8 disable_wbuffer;
 	u8 pngcprlevel_valid;
 	unsigned int pngcmprlevel;
 	void *zip_data;
@@ -667,6 +678,7 @@ void dbuf_read_to_ucstring_n(dbuf *f, i64 pos, i64 len, i64 max_len,
 #define DE_CREATEFLAG_IS_AUX   0x1
 #define DE_CREATEFLAG_OPT_IMAGE 0x2
 #define DE_CREATEFLAG_FLIP_IMAGE 0x4
+#define DE_CREATEFLAG_NO_WBUFFER 0x200
 dbuf *dbuf_create_output_file(deark *c, const char *ext, de_finfo *fi, unsigned int createflags);
 
 dbuf *dbuf_create_unmanaged_file(deark *c, const char *fname, int overwrite_mode, unsigned int flags);
@@ -683,7 +695,10 @@ dbuf *dbuf_create_membuf(deark *c, i64 initialsize, unsigned int flags);
 // If f is NULL, this is a no-op.
 void dbuf_close(dbuf *f);
 
+void dbuf_enable_wbuffer(dbuf *f);
+void dbuf_disable_wbuffer(dbuf *f);
 void dbuf_set_writelistener(dbuf *f, de_writelistener_cb_type fn, void *userdata);
+void de_writelistener_for_crc(dbuf *f, void *userdata, const u8 *buf, i64 buf_len);
 
 void dbuf_write(dbuf *f, const u8 *m, i64 len);
 void dbuf_write_at(dbuf *f, i64 pos, const u8 *m, i64 len);
@@ -711,6 +726,7 @@ void dbuf_writeu64le(dbuf *f, u64 n);
 void dbuf_puts(dbuf *f, const char *sz);
 void dbuf_printf(dbuf *f, const char *fmt, ...)
   de_gnuc_attribute ((format (printf, 2, 3)));
+void dbuf_flush_lowlevel(dbuf *f);
 void dbuf_flush(dbuf *f);
 
 // Read a slice of one dbuf, and append it to another dbuf.
@@ -753,6 +769,7 @@ int dbuf_dump_to_file(dbuf *inf, const char *fn);
 // May be valid only for memory buffers.
 void dbuf_empty(dbuf *f);
 
+i64 dbuf_get_length(dbuf *f);
 void dbuf_set_length_limit(dbuf *f, i64 max_len);
 const u8 *dbuf_get_membuf_direct_ptr(dbuf *f);
 int dbuf_search_byte(dbuf *f, const u8 b, i64 startpos, i64 haystack_len,
@@ -762,6 +779,8 @@ int dbuf_search(dbuf *f, const u8 *needle, i64 needle_len, i64 startpos,
 int dbuf_get_utf16_NULterm_len(dbuf *f, i64 pos1, i64 bytes_avail,
 	i64 *bytes_consumed);
 int dbuf_find_line(dbuf *f, i64 pos1, i64 *pcontent_len, i64 *ptotal_len);
+int de_memmatch(const u8 *mem, const u8 *pattern, size_t pattern_len,
+	u8 wildcard, UI flags);
 
 struct de_fourcc {
   u8 bytes[4];
@@ -789,6 +808,9 @@ int dbuf_buffered_read(dbuf *f, i64 pos, i64 len,
 int de_is_all_zeroes(const u8 *b, i64 n);
 int dbuf_is_all_zeroes(dbuf *f, i64 pos, i64 len);
 
+void dbuf_copy_slice_convert_to_utf8(dbuf *inf, i64 pos1, i64 len,
+	de_ext_encoding input_ee, dbuf *outf, UI flags);
+
 struct de_bitbuf_lowlevel {
 	u8 is_lsb;
 	UI nbits_in_bitbuf;
@@ -946,10 +968,15 @@ void de_make_grayscale_palette(de_color *pal, i64 num_entries, unsigned int flag
 char de_get_hexchar(int n);
 u8 de_decode_hex_digit(u8 x, int *errorflag);
 
-de_color de_palette_vga256(int index);
-de_color de_palette_ega64(int index);
-de_color de_palette_pc16(int index);
-de_color de_palette_pcpaint_cga4(int palnum, int index);
+#define DE_PALID_CGA    10
+#define DE_PALID_PC16   20
+#define DE_PALID_EGA64  30
+#define DE_PALID_VGA256 40
+
+de_color de_get_std_palette_entry(int pal_id, int pal_subid, int index);
+#define DE_COPYPALFLAG_INITPAL 0x10
+void de_copy_std_palette(int pal_id, int pal_subid, i64 starting_idx, i64 count,
+	de_color *dstpal, size_t dstpal_capacity, UI flags);
 
 const u8 *de_get_8x8ascii_font_ptr(void);
 const u8 *de_get_vga_cp437_font_ptr(void);
@@ -1023,6 +1050,7 @@ void ucstring_append_sz(de_ucstring *s, const char *sz, de_ext_encoding ee);
 
 void ucstring_write_as_utf8(deark *c, de_ucstring *s, dbuf *outf, int add_bom_if_needed);
 int de_is_printable_uchar(de_rune ch);
+int ucstring_is_ascii(const de_ucstring *s);
 i64 ucstring_count_utf8_bytes(de_ucstring *s);
 
 // Supported encodings are DE_ENCODING_UTF8, DE_ENCODING_ASCII, DE_ENCODING_LATIN1.
@@ -1078,10 +1106,9 @@ int de_inthashtable_remove_any_item(deark *c, struct de_inthashtable *ht, i64 *p
 
 #define DE_CRCOBJ_CRC32_IEEE   0x10
 #define DE_CRCOBJ_ADLER32      0x11
-#define DE_CRCOBJ_CRC16_CCITT  0x20
+#define DE_CRCOBJ_CRC16_XMODEM 0x20
 #define DE_CRCOBJ_CRC16_ARC    0x21
-
-struct de_crcobj;
+#define DE_CRCOBJ_CRC16_IBMSDLC 0x22
 
 struct de_crcobj *de_crcobj_create(deark *c, UI type_and_flags);
 void de_crcobj_destroy(struct de_crcobj *crco);
