@@ -217,6 +217,7 @@ int de_run(deark *c)
 	int moddisp;
 	int subdirs_opt;
 	int keepdirentries_opt;
+	int tmp_opt;
 	de_module_params *mparams = NULL;
 	de_ucstring *friendly_infn = NULL;
 
@@ -400,6 +401,20 @@ int de_run(deark *c)
 		c->max_output_files = DE_DEFAULT_MAX_OUTPUT_FILES;
 	}
 
+	if(de_get_ext_option_bool(c, "oinfo", 0)) {
+		c->enable_oinfo = 1;
+	}
+
+	tmp_opt = de_get_ext_option_bool(c, "wbuffer", -1);
+	if(tmp_opt>0) {
+		// For testing(?), enable wbuffer feature automatically in some cases.
+		c->enable_wbuffer_test = 1;
+	}
+	else if(tmp_opt==0) {
+		// Always disable wbuffer, even if a module wants to use it.
+		c->disable_wbuffer = 1;
+	}
+
 	// If we're writing to a zip file, we normally defer creating that zip file
 	// until we find a file to extract, so that we never create a zip file with
 	// no member files.
