@@ -1902,11 +1902,7 @@ static void adler32_continue(struct de_crcobj *crco, const u8 *buf, i64 buf_len)
 	crco->val = (s2 << 16) + s1;
 }
 
-// This is the CRC-16 algorithm used in MacBinary.
-// It is in the x^16 + x^12 + x^5 + 1 family.
-// CRC-16-CCITT is probably the best name for it, though I'm not completely
-// sure, and there are several algorithms that have been called "CRC-16-CCITT".
-static void de_crc16ccitt_init(struct de_crcobj *crco)
+static void de_crc16xmodem_init(struct de_crcobj *crco)
 {
 	const unsigned int polynomial = 0x1021;
 	unsigned int index;
@@ -1921,7 +1917,7 @@ static void de_crc16ccitt_init(struct de_crcobj *crco)
 	}
 }
 
-static void de_crc16ccitt_continue(struct de_crcobj *crco, const u8 *buf, i64 buf_len)
+static void de_crc16xmodem_continue(struct de_crcobj *crco, const u8 *buf, i64 buf_len)
 {
 	i64 k;
 
@@ -1932,11 +1928,7 @@ static void de_crc16ccitt_continue(struct de_crcobj *crco, const u8 *buf, i64 bu
 	}
 }
 
-// This is the CRC-16 algorithm used in ARC, LHA, ZOO, etc.
-// It is in the x^16 + x^15 + x^2 + 1 family.
-// It's some variant of CRC-16-IBM, and sometimes simply called "CRC-16". But
-// both these names are more ambiguous than I'd like, so I'm calling it "ARC".
-static void de_crc16arc_init(struct de_crcobj *crco)
+static void de_crc16arc_init(struct de_crcobj *crco, u16 poly)
 {
 	u32 i, k;
 
@@ -1944,7 +1936,7 @@ static void de_crc16arc_init(struct de_crcobj *crco)
 	for(i=0; i<256; i++) {
 		crco->table16[i] = i;
 		for(k=0; k<8; k++)
-			crco->table16[i] = (crco->table16[i]>>1) ^ ((crco->table16[i] & 1) ? 0xa001 : 0);
+			crco->table16[i] = (crco->table16[i]>>1) ^ ((crco->table16[i] & 1) ? poly : 0);
 	}
 }
 
@@ -1969,11 +1961,21 @@ struct de_crcobj *de_crcobj_create(deark *c, UI type_and_flags)
 	crco->crctype = type_and_flags;
 
 	switch(crco->crctype) {
-	case DE_CRCOBJ_CRC16_CCITT:
-		de_crc16ccitt_init(crco);
+	case DE_CRCOBJ_CRC16_XMODEM:
+		// This is the CRC-16 algorithm used in MacBinary.
+		// The CRC RevEng catalogue calls it "CRC-16/XMODEM".
+		de_crc16xmodem_init(crco);
 		break;
 	case DE_CRCOBJ_CRC16_ARC:
-		de_crc16arc_init(crco);
+		// This is the CRC-16 algorithm used in ARC, LHA, ZOO, etc.
+		// The CRC RevEng catalogue calls it "CRC-16/ARC".
+		de_crc16arc_init(crco, 0xa001);
+		break;
+	case DE_CRCOBJ_CRC16_IBMSDLC:
+		// This is the CRC-16 algorithm used in ar001.
+		// I'm pretty sure it is equivalent to the one the CRC RevEng catalogue
+		// calls "CRC-16/IBM-SDLC".
+		de_crc16arc_init(crco, 0x8408);
 		break;
 	}
 
@@ -2002,11 +2004,18 @@ void de_crcobj_reset(struct de_crcobj *crco)
 	case DE_CRCOBJ_ADLER32:
 		crco->val = 1;
 		break;
+	case DE_CRCOBJ_CRC16_IBMSDLC:
+		crco->val = 0xffff;
+		break;
 	}
 }
 
 u32 de_crcobj_getval(struct de_crcobj *crco)
 {
+	switch(crco->crctype) {
+	case DE_CRCOBJ_CRC16_IBMSDLC:
+		return crco->val ^ 0xffff;
+	}
 	return crco->val;
 }
 
@@ -2018,10 +2027,11 @@ void de_crcobj_addbuf(struct de_crcobj *crco, const u8 *buf, i64 buf_len)
 	case DE_CRCOBJ_CRC32_IEEE:
 		crco->val = de_crc32_continue(crco->val, buf, buf_len);
 		break;
-	case DE_CRCOBJ_CRC16_CCITT:
-		de_crc16ccitt_continue(crco, buf, buf_len);
+	case DE_CRCOBJ_CRC16_XMODEM:
+		de_crc16xmodem_continue(crco, buf, buf_len);
 		break;
 	case DE_CRCOBJ_CRC16_ARC:
+	case DE_CRCOBJ_CRC16_IBMSDLC:
 		de_crc16arc_continue(crco, buf, buf_len);
 		break;
 	case DE_CRCOBJ_ADLER32:
@@ -2077,3 +2087,18 @@ int de_archive_initialize(deark *c)
 	}
 	return 0;
 }
+
+int de_memmatch(const u8 *mem, const u8 *pattern, size_t pattern_len,
+	u8 wildcard, UI flags)
+{
+	size_t i;
+
+	for(i=0; i<pattern_len; i++) {
+		u8 m = mem[i];
+		u8 p = pattern[i];
+
+		if(p==wildcard) continue;
+		if(p!=m) return 0;
+	}
+	return 1;
+}
