@@ -39,6 +39,7 @@ void fmtutil_get_bmp_compression_name(u32 code, char *s, size_t s_len,
 // Otherwise, it points to the BITMAPINFOHEADER.
 // Caller allocates bi.
 // Returns 0 if BMP is invalid.
+// If DE_BMPINFO_NOERR is set, will not report errors, but may still return 0.
 int fmtutil_get_bmpinfo(deark *c, dbuf *f, struct de_bmpinfo *bi, i64 pos,
 	i64 len, unsigned int flags)
 {
@@ -158,8 +159,15 @@ int fmtutil_get_bmpinfo(deark *c, dbuf *f, struct de_bmpinfo *bi, i64 pos,
 	bi->is_compressed = !((bi->compression_field==0) ||
 		(bi->compression_field==3 && bi->bitcount>1));
 
-	if(!de_good_image_dimensions(c, bi->width, bi->height)) {
-		return 0;
+	if(flags & DE_BMPINFO_NOERR) {
+		if(!de_good_image_dimensions_noerr(c, bi->width, bi->height)) {
+			return 0;
+		}
+	}
+	else {
+		if(!de_good_image_dimensions(c, bi->width, bi->height)) {
+			return 0;
+		}
 	}
 
 	// TODO: This needs work, to decide how to handle compressed images.
@@ -920,7 +928,7 @@ void fmtutil_atari_set_standard_density(deark *c, struct atari_img_decode_data *
 
 void fmtutil_atari_help_palbits(deark *c)
 {
-	de_msg(c, "-opt atari:palbits=<9|12|15> : Numer of significant bits "
+	de_msg(c, "-opt atari:palbits=<9|12|15> : Number of significant bits "
 		"per palette color");
 }
 
@@ -1449,3 +1457,640 @@ i64 fmtutil_hlp_get_csl_p(dbuf *f, i64 *ppos)
 	x1 -= 67108864;
 	return x1;
 }
+
+struct execomp_ctx {
+	u8 devmode;
+	char shortname[40];
+	char verstr[40];
+};
+
+static void calc_entrypoint_crc(deark *c, struct execomp_ctx *ectx, struct fmtutil_exe_info *ei)
+{
+	struct de_crcobj *crco = NULL;
+	u32 crc1, crc2;
+
+	if(ei->entrypoint_crcs!=0) return;
+
+	// Sniff some bytes, starting at the code entry point.
+	crco = de_crcobj_create(c, DE_CRCOBJ_CRC32_IEEE);
+	de_crcobj_addslice(crco, ei->f, ei->entry_point, 32);
+	crc1 = de_crcobj_getval(crco);
+	de_crcobj_reset(crco);
+	de_crcobj_addslice(crco, ei->f, ei->entry_point+32, 32);
+	crc2 = de_crcobj_getval(crco);
+	ei->entrypoint_crcs = ((u64)crc1 << 32) | crc2;
+	if(ectx->devmode) {
+		de_dbg(c, "execomp crc: %016"U64_FMTx, ei->entrypoint_crcs);
+	}
+
+	de_crcobj_destroy(crco);
+}
+
+static void detect_execomp_pklite(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	u8 maybe_pklite = 0; // Any format with the usual EXE header field values
+	u8 maybe_beta = 0;
+	u8 maybe_megalite = 0;
+	u8 cb[12]; // Bytes at entry point
+
+	if(ei->regIP==256 && ei->regCS==(-16) && ei->num_relocs<=2 &&
+		ei->entry_point==ei->start_of_dos_code)
+	{
+		maybe_pklite = 1;
+	}
+	else if(ei->regIP==256 && ei->regCS>(-16) && ei->num_relocs==0 &&
+		ei->entry_point>ei->start_of_dos_code)
+	{
+		maybe_beta = 1;
+	}
+	else if(ei->regCS==0 && ei->regIP==0 && ei->num_relocs==1 && ei->reloc_table_pos==28 &&
+		ei->entry_point==ei->start_of_dos_code)
+	{
+		maybe_megalite = 1;
+	}
+	else {
+		goto done;
+	}
+
+	dbuf_read(ei->f, cb, ei->entry_point, sizeof(cb));
+
+	if(maybe_pklite) {
+		if(de_memmatch(cb, (const u8*)"\xb8??\xba??\x8c\xdb\x03\xd8\x3b", 11, '?', 0)) {
+			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // v1.00-1.05, etc.
+		}
+		if(de_memmatch(cb, (const u8*)"\xb8??\xba??\x05\x00\x00\x3b", 10, '?', 0)) {
+			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // v1.12-1.15, etc.
+		}
+		if(de_memmatch(cb, (const u8*)"\x50\xb8??\xba??\x05\x00\x00\x3b", 11, '?', 0)) {
+			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // v2.01, etc.
+		}
+		if(de_memmatch(cb, (const u8*)"\x9c\xba?\x00\x2d?\x00\x81\xe1?\x00\x81", 12, '?', 0)) {
+			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // Patched by UN^2PACK v2.0?
+		}
+		goto done;
+	}
+
+	if(maybe_beta) {
+		if(de_memmatch(cb, (const u8*)"\x2e\x8c\x1e??\x8b\x1e\x02\x00\x8c\xda\x81", 12, '?', 0)) {
+			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // Normal beta
+		}
+		if(de_memmatch(cb, (const u8*)"\x2e\x8c\x1e??\xfc\x8c\xc8\x2e\x2b\x06", 11, '?', 0)) {
+			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // beta/loadhigh
+		}
+		goto done;
+	}
+
+	if(maybe_megalite) {
+		if(de_memmatch(cb, (const u8*)"\xb8??\xba??\x05\x00\x00\x3b\x2d", 11, '?', 0)) {
+			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE;
+		}
+		goto done;
+	}
+
+done:
+	if(edd->detected_fmt==DE_SPECIALEXEFMT_PKLITE) {
+		de_strlcpy(ectx->shortname, "PKLITE", sizeof(ectx->shortname));
+		edd->modname = "pklite";
+	}
+}
+
+static void detect_execomp_lzexe(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	const char *vs = NULL;
+
+	if(ei->entrypoint_crcs==0x4b6802c9cf419437LLU) {
+		edd->detected_subfmt = 1;
+		vs = "0.90";
+	}
+	else if(ei->entrypoint_crcs==0x246655c50ae99574LLU) {
+		edd->detected_subfmt = 2;
+		vs = "0.91";
+	}
+	else if(ei->entrypoint_crcs==0xd8a60f138f680f0cLLU) {
+		edd->detected_subfmt = 3;
+		vs = "0.91e";
+	}
+
+	if(vs) {
+		edd->detected_fmt = DE_SPECIALEXEFMT_LZEXE;
+		de_strlcpy(ectx->shortname, "LZEXE", sizeof(ectx->shortname));
+		de_strlcpy(ectx->verstr, vs, sizeof(ectx->verstr));
+		edd->modname = "lzexe";
+	}
+}
+
+static void detect_execomp_exepack(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	u8 x, x2;
+	int has_RB = 0;
+
+	if(ei->num_relocs!=0) goto done;
+	if(ei->regIP!=16 && ei->regIP!=18) goto done;
+
+	if(dbuf_getu16be(ei->f, ei->entry_point-2) == 0x5242) {
+		has_RB = 1;
+	}
+
+	if(ei->entrypoint_crcs==0xa6ea214e6c16ee72LLU) {
+		edd->detected_subfmt = 5;
+		goto done;
+	}
+	else if(ei->entrypoint_crcs==0x4e04abaac5d3b465LLU) {
+		edd->detected_subfmt = 4;
+		goto done;
+	}
+	else if(ei->entrypoint_crcs==0x1f449ca73852e197LLU) {
+		x = dbuf_getbyte(ei->f, ei->entry_point+73);
+		x2 = dbuf_getbyte(ei->f, ei->entry_point+137);
+		if(x==0x0a && x2==0x27) {
+			edd->detected_subfmt = 2;
+			goto done;
+		}
+		else if(x==0x0a) {
+			edd->detected_subfmt = 1;
+			goto done;
+		}
+		else if(x==0x09) {
+			edd->detected_subfmt = 3;
+			goto done;
+		}
+	}
+	else if(ei->entrypoint_crcs==0x1da67457b559a299LLU) {
+		edd->detected_subfmt = 10; // David Fifield's exepack v1.3.0
+	}
+	else if(ei->entrypoint_crcs==0x629aca0c6520250aLLU) {
+		// EXPAKFIX-patched file. Original variant undetermined (TODO?).
+		edd->detected_subfmt = 11;
+	}
+
+	// TODO: Is SP always 128?
+	if(ei->regSP==128 && has_RB) {
+		edd->detected_fmt = DE_SPECIALEXEFMT_EXEPACK;
+		goto done;
+	}
+
+done:
+	if(edd->detected_subfmt!=0) {
+		edd->detected_fmt = DE_SPECIALEXEFMT_EXEPACK;
+	}
+
+	if(edd->detected_fmt==DE_SPECIALEXEFMT_EXEPACK) {
+		if(ectx->devmode) {
+			de_dbg(c, "epvar: %u", (UI)edd->detected_subfmt);
+		}
+		de_strlcpy(ectx->shortname, "EXEPACK", sizeof(ectx->shortname));
+		edd->modname = "exepack";
+	}
+}
+
+static void detect_execomp_tinyprog(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	i64 pos;
+	i64 j;
+	u8 x;
+
+	if(ei->num_relocs!=0) goto done;
+	pos = ei->entry_point;
+	x = dbuf_getbyte(ei->f, pos);
+	if(x!=0xe9) goto done;
+	j = dbuf_getu16le(ei->f, pos+1);
+	pos += 3+j; // Jump over user data
+
+	x = dbuf_getbyte(ei->f, pos);
+	if(x != 0xeb) goto done;
+	j = dbuf_geti8(ei->f, pos+1);
+	pos += 2+j; // Jump over (some sort of) data
+
+	// This part seems consistent, but I'm really just guessing. Need to test more versions.
+	if(dbuf_memcmp(ei->f, pos,
+		(const void*)"\x83\xec\x10\x83\xe4\xe0\x8b\xec\x50\xbe\x05\x01\x03\x36\x01\x01", 16))
+	{
+		goto done;
+	}
+
+	edd->detected_fmt = DE_SPECIALEXEFMT_TINYPROG;
+	de_strlcpy(ectx->shortname, "TINYPROG", sizeof(ectx->shortname));
+done:
+	;
+}
+
+static int execomp_diet_check_fingerprint(dbuf *f, i64 pos)
+{
+	u8 x;
+
+	if(dbuf_memcmp(f, pos,
+		(const u8*)"\x8e\xdb\x8e\xc0\x33\xf6\x33\xff\xb9\x08\x00\xf3\xa5\x4b\x48\x4a", 16))
+	{
+		return 0;
+	}
+
+	// Attempt to screen out LGLZ-compressed files (x==0x95).
+	x = dbuf_getbyte(f, pos+26);
+	if(x==0x8b) return 1;
+	return 0;
+}
+
+static void detect_execomp_diet(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	static const u8 offsets[] = {20, 40, 45};
+	i64 foundpos = 0;
+	size_t i;
+
+	if(ei->regCS!=0) goto done;
+	if(ei->regIP!=0 && ei->regIP!=3) goto done;
+	if(ei->num_relocs>1) goto done;
+
+	// Haven't figured out a good way to detect DIET. More research needed.
+	for(i=0; i<DE_ARRAYCOUNT(offsets); i++) {
+		if(execomp_diet_check_fingerprint(ei->f, ei->entry_point+(i64)offsets[i])) {
+			foundpos = (i64)offsets[i];
+			break;
+		}
+	}
+	if(foundpos==0) goto done;
+
+	edd->detected_fmt = DE_SPECIALEXEFMT_DIET;
+	de_strlcpy(ectx->shortname, "DIET", sizeof(ectx->shortname));
+done:
+	;
+}
+
+static void detect_execomp_wwpack(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	u8 cb[11]; // Bytes at entry point
+
+	if(ei->num_relocs!=0) goto done;
+	if(ei->start_of_dos_code!=32) goto done;
+	if(ei->entry_point==ei->start_of_dos_code) goto done;
+
+	dbuf_read(ei->f, cb, ei->entry_point, sizeof(cb));
+	if(!de_memmatch(cb, (const u8*)"\xb8??\x8c\xca\x03\xd0\x8c\xc9\x81\xc1", 11, '?', 0)) {
+		goto done;
+	}
+
+	edd->detected_fmt = DE_SPECIALEXEFMT_EXECOMP;
+	de_strlcpy(ectx->shortname, "WWPACK", sizeof(ectx->shortname));
+done:
+	;
+}
+
+// Caller initializes ei (to zeroes).
+// Records some basic information about an EXE file, to be used by routines that
+// detect special EXE formats.
+// The input file (f) must stay open after this. The detection routines will need
+// to read more of it.
+void fmtutil_collect_exe_info(deark *c, dbuf *f, struct fmtutil_exe_info *ei)
+{
+	i64 hdrsize; // in 16-byte units
+	i64 lfb, nblocks;
+
+	ei->f = f;
+	lfb = dbuf_getu16le(f, 2);
+	nblocks = dbuf_getu16le(f, 4);
+	ei->num_relocs = dbuf_getu16le(f, 6);
+	hdrsize = dbuf_getu16le(f, 8);
+	ei->start_of_dos_code = hdrsize*16;
+	ei->regSS = dbuf_geti16le(f, 14);
+	ei->regSP = dbuf_getu16le(f, 16);
+	ei->regIP = dbuf_getu16le(f, 20);
+	ei->regCS = dbuf_geti16le(f, 22);
+	ei->reloc_table_pos = dbuf_getu16le(f, 24);
+	ei->entry_point = (hdrsize + ei->regCS)*16 + ei->regIP;
+
+	ei->end_of_dos_code = nblocks*512;
+	if(lfb>=1 && lfb<=511) {
+		ei->end_of_dos_code = ei->end_of_dos_code - 512 + lfb;
+	}
+}
+
+// Caller supplies ei -- must call fmtutil_collect_exe_info() first.
+// Caller initializes edd, to receive the results.
+// If success, sets edd->detected_fmt to nonzero.
+// Always sets edd->detected_fmt_name to something, even if "unknown".
+// If we think we can decompress the format, sets edd->modname.
+void fmtutil_detect_execomp(deark *c, struct fmtutil_exe_info *ei,
+	struct fmtutil_specialexe_detection_data *edd)
+{
+	struct execomp_ctx ectx;
+
+	de_zeromem(&ectx, sizeof(struct execomp_ctx));
+	edd->detected_fmt = 0;
+	edd->detected_subfmt = 0;
+
+	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_PKLITE) {
+		detect_execomp_pklite(c, &ectx, ei, edd);
+		if(edd->detected_fmt!=0) goto done;
+	}
+
+	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_TINYPROG) {
+		detect_execomp_tinyprog(c, &ectx, ei, edd);
+		if(edd->detected_fmt!=0) goto done;
+	}
+
+	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_DIET) {
+		detect_execomp_diet(c, &ectx, ei, edd);
+		if(edd->detected_fmt!=0) goto done;
+	}
+
+	if(edd->restrict_to_fmt==0) {
+		detect_execomp_wwpack(c, &ectx, ei, edd);
+		if(edd->detected_fmt!=0) goto done;
+	}
+
+	calc_entrypoint_crc(c, &ectx, ei);
+
+	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_LZEXE) {
+		detect_execomp_lzexe(c, &ectx, ei, edd);
+		if(edd->detected_fmt!=0) goto done;
+	}
+
+	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_EXEPACK) {
+		detect_execomp_exepack(c, &ectx, ei, edd);
+		if(edd->detected_fmt!=0) goto done;
+	}
+
+done:
+	if(ectx.shortname[0] && ectx.verstr[0]) {
+		de_snprintf(edd->detected_fmt_name, sizeof(edd->detected_fmt_name), "%s %s",
+			ectx.shortname, ectx.verstr);
+	}
+	else {
+		de_strlcpy(edd->detected_fmt_name, (ectx.shortname[0]?ectx.shortname:"unknown"),
+			sizeof(edd->detected_fmt_name));
+	}
+}
+
+// If found, writes a copy of pos to *pfoundpos.
+static int is_lhalike_data_at(struct fmtutil_exe_info *ei, i64 pos, u8 h_or_z, i64 *pfoundpos)
+{
+	u8 b2[8];
+
+	if(pos+21 > ei->f->len) return 0;
+	dbuf_read(ei->f, b2, pos, sizeof(b2));
+	if(b2[2]!='-' || b2[6]!='-') return 0;
+	if(b2[3]=='l' && b2[4]==h_or_z) {
+		*pfoundpos = pos;
+		return 1;
+	}
+	return 0;
+}
+
+static int is_lha_data_at(struct fmtutil_exe_info *ei, i64 pos, i64 *pfoundpos)
+{
+	return is_lhalike_data_at(ei, pos, 'h', pfoundpos);
+}
+
+// Detect LHA/LHarc self-extracting DOS EXE formats.
+// TODO: This is a work in progress.
+static void detect_exesfx_lha(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	u8 b[16];
+	int found;
+	i64 foundpos = 0;
+
+	if(ei->regSS != -16) goto done;
+	if(ei->regSP != 256) goto done;
+	if(ei->regCS != -16) goto done;
+	if(ei->regIP != 256) goto done;
+
+	dbuf_read(ei->f, b, 28, sizeof(b));
+	if(b[4]!=0xeb && b[4]!=0xe9) goto done;
+	if(b[8]=='L' && b[9]=='H') {
+		;
+	}
+	else if(b[9]=='L' && b[10]=='H') {
+		;
+	}
+	else if(b[10]=='L' && b[11]=='H') {
+		;
+	}
+	else if(b[10]=='S' && b[11]=='F') { // v1.00
+		;
+	}
+	else {
+		goto done;
+	}
+
+	found =
+		is_lha_data_at(ei, ei->end_of_dos_code, &foundpos) ||
+		is_lha_data_at(ei, ei->end_of_dos_code+1, &foundpos) ||
+		is_lha_data_at(ei, ei->end_of_dos_code+3, &foundpos) ||
+		is_lha_data_at(ei, ei->entry_point + 1292-32, &foundpos) ||
+		is_lha_data_at(ei, ei->entry_point + 1295-32, &foundpos) ||
+		is_lha_data_at(ei, ei->entry_point + 1322-32, &foundpos);
+	if(!found) goto done;
+
+	edd->payload_pos = foundpos;
+	edd->payload_len = ei->f->len - edd->payload_pos;
+	if(edd->payload_len<21) goto done;
+
+	edd->detected_fmt = DE_SPECIALEXEFMT_SFX;
+	edd->payload_valid = 1;
+	edd->payload_file_ext = "lha";
+
+done:
+	if(edd->detected_fmt==DE_SPECIALEXEFMT_SFX) {
+		de_strlcpy(ectx->shortname, "LHA", sizeof(ectx->shortname));
+	}
+}
+
+static void detect_exesfx_larc(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	int found;
+	i64 foundpos = 0;
+
+	calc_entrypoint_crc(c, ectx, ei);
+	if(ei->entrypoint_crcs!=0x81681852d3882b18ULL) {
+		goto done;
+	}
+
+	found = is_lhalike_data_at(ei, ei->entry_point+525, 'z', &foundpos);
+	if(!found) goto done;
+
+	edd->payload_pos = foundpos;
+	edd->payload_len = ei->f->len - edd->payload_pos;
+	if(edd->payload_len<21) goto done;
+
+	edd->detected_fmt = DE_SPECIALEXEFMT_SFX;
+	edd->payload_valid = 1;
+	edd->payload_file_ext = "lzs";
+
+done:
+	if(edd->detected_fmt==DE_SPECIALEXEFMT_SFX) {
+		de_strlcpy(ectx->shortname, "LArc", sizeof(ectx->shortname));
+	}
+}
+
+static int is_arc_data_at(struct fmtutil_exe_info *ei, i64 pos)
+{
+	u8 b[2];
+
+	dbuf_read(ei->f, b, pos, 2);
+	if(b[0]!=0x1a) return 0;
+	if(b[1]>30) return 0;
+	return 1;
+}
+
+// Detect some ARC self-extracting DOS EXE formats.
+// TODO: This is pretty fragile. It only detects files made by known versions of
+// MKSARC (from the ARC distribution).
+static void detect_exesfx_arc(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	int found = 0;
+	i64 foundpos = 0;
+
+	calc_entrypoint_crc(c, ectx, ei);
+	if(ei->entrypoint_crcs==0x057db6d8be3c4895ULL) { // MKSARC v1.00 from ARC v6.01
+		found = 1;
+		foundpos = ei->end_of_dos_code;
+	}
+	else if(ei->entrypoint_crcs==0x8542182a98613fe0ULL ||
+		ei->entrypoint_crcs==0x6bd653c8edd98eedULL)
+	{
+		// MKSARC v1.01 from ARC v6.02
+		// (Found two different versions with the same version number.)
+		// This version of MKSARC has a bug. Compared to the v1.00, the start of DOS
+		// code was reduced by 480, from 512 to 32. But the *end* of DOS code was not
+		// adjusted accordingly, leaving it 480 higher than it should be. This is
+		// important, because the end of DOS code is where we expect the ARC data to
+		// start.
+		found = 1;
+		foundpos = ei->end_of_dos_code - 480;
+		if(!is_arc_data_at(ei, foundpos)) {
+			foundpos = ei->end_of_dos_code; // In case there's a version without the bug
+		}
+	}
+	else if(ei->entrypoint_crcs==0x3230b4d5fca84644ULL) { // MKSARC v7.12, from ARC v7.12
+		found = 1;
+		foundpos = ei->end_of_dos_code;
+	}
+	// TODO: Detect MKSARC v7.12 with the /P option (OS/2 protected mode).
+	// Extraction would work, if we could detect it.
+
+	if(!found) goto done;
+	if(!is_arc_data_at(ei, foundpos)) {
+		goto done;
+	}
+
+	edd->payload_pos = foundpos;
+	edd->payload_len = ei->f->len - edd->payload_pos;
+	if(edd->payload_len<2) goto done;
+	// TODO: It would be nice to strip any padding from the end of the extracted ARC
+	// file, but that could be more trouble than it's worth.
+
+	edd->detected_fmt = DE_SPECIALEXEFMT_SFX;
+	edd->payload_valid = 1;
+	edd->payload_file_ext = "arc";
+
+done:
+	if(edd->detected_fmt==DE_SPECIALEXEFMT_SFX) {
+		de_strlcpy(ectx->shortname, "ARC", sizeof(ectx->shortname));
+	}
+}
+
+// Also uses/updates edd->zip_eocd_* settings.
+static void detect_exesfx_zip(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	int ret;
+	i64 foundpos = 0;
+
+	if(!edd->zip_eocd_looked_for) {
+		if(c->infile==ei->f && c->detection_data && c->detection_data->zip_eocd_looked_for) {
+			edd->zip_eocd_looked_for = 1;
+			edd->zip_eocd_found = c->detection_data->zip_eocd_found;
+			edd->zip_eocd_pos = c->detection_data->zip_eocd_pos;
+		}
+		else {
+			edd->zip_eocd_found = (u8)fmtutil_find_zip_eocd(c, c->infile, &edd->zip_eocd_pos);
+			edd->zip_eocd_looked_for = 1;
+		}
+	}
+
+	if(!edd->zip_eocd_found) goto done;
+
+	// Look for a ZIP file in the overlay (approximately)
+	if(ei->f->len - ei->end_of_dos_code < 10) goto done; // Overlay too small
+
+	ret = dbuf_search(ei->f, (const u8*)"PK\x03\x04", 4, ei->end_of_dos_code-10, 20,
+		&foundpos);
+	if(!ret) goto done;
+	edd->detected_fmt = DE_SPECIALEXEFMT_ZIPSFX;
+
+done:
+	if(edd->detected_fmt==DE_SPECIALEXEFMT_ZIPSFX) {
+		de_strlcpy(ectx->shortname, "ZIP", sizeof(ectx->shortname));
+	}
+}
+
+static void detect_exesfx_zoo(deark *c, struct execomp_ctx *ectx,
+	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+{
+	i64 overlay_len;
+
+	overlay_len = ei->f->len - ei->end_of_dos_code;
+	if(overlay_len < 24) goto done;
+	if((UI)dbuf_getu32le(ei->f, ei->end_of_dos_code+20) != 0xfdc4a7dc) goto done;
+
+	calc_entrypoint_crc(c, ectx, ei);
+	switch((UI)(ei->entrypoint_crcs>>32)) {
+	case 0x6d384fa1U: // SEZ 2.00
+	case 0xec5138deU: // SEZ 2.30
+		// TODO: More Zoo SFX variants
+		break;
+	default:
+		goto done;
+	}
+
+	edd->payload_pos = ei->end_of_dos_code;
+	edd->payload_len = overlay_len;
+	edd->payload_valid = 1;
+	edd->detected_fmt = DE_SPECIALEXEFMT_SFX;
+	edd->payload_file_ext = "zoo";
+
+done:
+	if(edd->detected_fmt==DE_SPECIALEXEFMT_SFX) {
+		de_strlcpy(ectx->shortname, "Zoo", sizeof(ectx->shortname));
+	}
+}
+
+void fmtutil_detect_exesfx(deark *c, struct fmtutil_exe_info *ei,
+	struct fmtutil_specialexe_detection_data *edd)
+{
+	struct execomp_ctx ectx;
+
+	de_zeromem(&ectx, sizeof(struct execomp_ctx));
+
+	detect_exesfx_zip(c, &ectx, ei, edd);
+	if(edd->detected_fmt) goto done;
+
+	detect_exesfx_lha(c, &ectx, ei, edd);
+	if(edd->detected_fmt) goto done;
+
+	detect_exesfx_arc(c, &ectx, ei, edd);
+	if(edd->detected_fmt) goto done;
+
+	detect_exesfx_larc(c, &ectx, ei, edd);
+	if(edd->detected_fmt) goto done;
+
+	detect_exesfx_zoo(c, &ectx, ei, edd);
+
+done:
+	if(ectx.shortname[0] && ectx.verstr[0]) {
+		de_snprintf(edd->detected_fmt_name, sizeof(edd->detected_fmt_name), "%s %s",
+			ectx.shortname, ectx.verstr);
+	}
+	else {
+		de_strlcpy(edd->detected_fmt_name, (ectx.shortname[0]?ectx.shortname:"unknown"),
+			sizeof(edd->detected_fmt_name));
+	}
+}
